"""
Contains Enums for Prognosegrundlage.
"""

from bo4e.enum.strenum import StrEnum


class Prognosegrundlage(StrEnum):
    """
    Prognosegrundlage (WERTE, PROFILE).
    """

    WERTE = "WERTE"  #: Prognose auf Basis von Werten
    PROFILE = "PROFILE"  #: Prognose auf Basis von Profilen



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Kontaktart(StrEnum):
    """
    Gibt an, auf welchem Weg die Person oder der Geschäftspartner kontaktiert werden kann.
    """

    POSTWEG = "POSTWEG"
    """Postweg"""
    TELEFON = "TELEFON"
    """Telefon"""
    FAX = "FAX"
    """Fax"""
    E_MAIL = "E_MAIL"
    """E-Mail"""
    SMS = "SMS"
    """SMS"""



"""
Contains Ausschreibungslos class
"""

# pylint: disable=too-few-public-methods, too-many-instance-attributes
# pylint: disable=no-name-in-module
from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.preismodell import Preismodell
    from ..enum.rechnungslegung import Rechnungslegung
    from ..enum.sparte import Sparte
    from ..enum.vertragsform import Vertragsform
    from .ausschreibungsdetail import Ausschreibungsdetail
    from .menge import Menge
    from .zeitraum import Zeitraum


@postprocess_docstring
class Ausschreibungslos(COM):
    """
    Eine Komponente zur Abbildung einzelner Lose einer Ausschreibung

    .. raw:: html

        <object data="../_static/images/bo4e/com/Ausschreibungslos.svg" type="image/svg+xml"></object>

    .. HINT::
        `Ausschreibungslos JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Ausschreibungslos.json>`_

    """

    typ: Annotated[Literal[ComTyp.AUSSCHREIBUNGSLOS], Field(alias="_typ")] = ComTyp.AUSSCHREIBUNGSLOS

    losnummer: Optional[str] = None
    """Laufende Nummer des Loses"""
    bezeichnung: Optional[str] = None
    """Bezeichnung der Ausschreibung"""
    preismodell: Optional["Preismodell"] = None
    """Bezeichnung der Preismodelle in Ausschreibungen für die Energielieferung"""

    energieart: Optional["Sparte"] = None
    """Unterscheidungsmöglichkeiten für die Sparte"""
    wunsch_rechnungslegung: Optional["Rechnungslegung"] = None
    """Aufzählung der Möglichkeiten zur Rechnungslegung in Ausschreibungen"""
    wunsch_vertragsform: Optional["Vertragsform"] = None
    """Aufzählung der Möglichkeiten zu Vertragsformen in Ausschreibungen"""
    betreut_durch: Optional[str] = None
    """Name des Lizenzpartners"""
    anzahl_lieferstellen: Optional[int] = None
    """Anzahl der Lieferstellen in dieser Ausschreibung"""

    lieferstellen: Optional[list["Ausschreibungsdetail"]] = None
    """Die ausgeschriebenen Lieferstellen"""

    lieferzeitraum: Optional["Zeitraum"] = None
    """Zeitraum, für den die in diesem Los enthaltenen Lieferstellen beliefert werden sollen"""

    bemerkung: Optional[str] = None
    """Bemerkung des Kunden zum Los"""
    gesamt_menge: Optional["Menge"] = None
    """Gibt den Gesamtjahresverbrauch (z.B. in kWh) aller in diesem Los enthaltenen Lieferstellen an"""
    wunsch_mindestmenge: Optional["Menge"] = None
    """Mindesmenge Toleranzband (kWh, %)"""
    wunsch_maximalmenge: Optional["Menge"] = None
    """Maximalmenge Toleranzband (kWh, %)"""

    wiederholungsintervall: Optional["Zeitraum"] = None
    """
    In welchem Intervall die Angebotsabgabe wiederholt werden darf.
    Angabe nur gesetzt für die 2. Phase bei öffentlich-rechtlichen Ausschreibungen
    """

    wunsch_kuendingungsfrist: Optional["Zeitraum"] = None
    """Kundenwunsch zur Kündigungsfrist in der Ausschreibung"""
    wunsch_zahlungsziel: Optional["Zeitraum"] = None
    """Kundenwunsch zum Zahlungsziel in der Ausschreibung"""



"""
Contains Angebotsvariante
"""

# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module
from typing import TYPE_CHECKING, Annotated, Literal, Optional

import pydantic
from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.angebotsstatus import Angebotsstatus
    from .angebotsteil import Angebotsteil
    from .betrag import Betrag
    from .menge import Menge


@postprocess_docstring
class Angebotsvariante(COM):
    """
    Führt die verschiedenen Ausprägungen der Angebotsberechnung auf

    .. raw:: html

        <object data="../_static/images/bo4e/com/Angebotsvariante.svg" type="image/svg+xml"></object>

    .. HINT::
        `Angebotsvariante JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Angebotsvariante.json>`_

    """

    typ: Annotated[Literal[ComTyp.ANGEBOTSVARIANTE], Field(alias="_typ")] = ComTyp.ANGEBOTSVARIANTE

    angebotsstatus: Optional["Angebotsstatus"] = None
    """Gibt den Status eines Angebotes an."""

    erstellungsdatum: Optional[pydantic.AwareDatetime] = None
    """Datum der Erstellung der Angebotsvariante"""

    bindefrist: Optional[pydantic.AwareDatetime] = None
    """Bis zu diesem Zeitpunkt gilt die Angebotsvariante"""

    teile: Optional[list["Angebotsteil"]] = None
    """
    Angebotsteile werden im einfachsten Fall für eine Marktlokation oder Lieferstellenadresse erzeugt.
    Hier werden die Mengen und Gesamtkosten aller Angebotspositionen zusammengefasst.
    Eine Variante besteht mindestens aus einem Angebotsteil.
    """

    gesamtmenge: Optional["Menge"] = None
    """Aufsummierte Wirkarbeitsmenge aller Angebotsteile"""
    # todo: write a validator for this: https://github.com/Hochfrequenz/BO4E-python/issues/320
    gesamtkosten: Optional["Betrag"] = None
    """Aufsummierte Kosten aller Angebotsteile"""



"""
Contains Fremdkosten class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

if TYPE_CHECKING:
    from ..com.betrag import Betrag
    from ..com.fremdkostenblock import Fremdkostenblock
    from ..com.zeitraum import Zeitraum


# pylint: disable=too-few-public-methods


@postprocess_docstring
class Fremdkosten(Geschaeftsobjekt):
    """
    Mit diesem BO werden die Fremdkosten, beispielsweise für eine Angebotserstellung oder eine Rechnungsprüfung,
    übertragen.
    Die Fremdkosten enthalten dabei alle Kostenblöcke, die von anderen Marktteilnehmern oder Instanzen erhoben werden.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Fremdkosten.svg" type="image/svg+xml"></object>

    .. HINT::
        `Fremdkosten JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Fremdkosten.json>`_

    """

    typ: Annotated[Literal[BoTyp.FREMDKOSTEN], Field(alias="_typ")] = BoTyp.FREMDKOSTEN
    gueltigkeit: Optional["Zeitraum"] = None
    """Für diesen Zeitraum wurden die Kosten ermittelt"""
    summe_kosten: Optional["Betrag"] = None
    """Die Gesamtsumme über alle Kostenblöcke und -positionen"""
    kostenbloecke: Optional[list["Fremdkostenblock"]] = None
    """In Kostenblöcken werden Kostenpositionen zusammengefasst. Beispiele: Netzkosten, Umlagen, Steuern etc"""



# pylint: disable=missing-module-docstring, line-too-long
from bo4e.enum.strenum import StrEnum


class Rechnungstyp(StrEnum):
    """
    Abbildung verschiedener Rechnungstypen zur Kennzeichnung von Rechnungen
    """

    ENDKUNDENRECHNUNG = "ENDKUNDENRECHNUNG"
    """Eine Rechnung vom Lieferanten an einen Endkunden über die Lieferung von Energie."""
    NETZNUTZUNGSRECHNUNG = "NETZNUTZUNGSRECHNUNG"
    """Eine Rechnung vom Netzbetreiber an den Netznutzer. (i.d.R. der Lieferant) über die Netznutzung."""
    MEHRMINDERMENGENRECHNUNG = "MEHRMINDERMENGENRECHNUNG"
    """Eine Rechnung vom Netzbetreiber an den Netznutzer. (i.d.R. der Lieferant) zur Abrechnung von Mengen-Differenzen zwischen Bilanzierung und Messung."""
    MESSSTELLENBETRIEBSRECHNUNG = "MESSSTELLENBETRIEBSRECHNUNG"
    """Rechnung eines Messstellenbetreibers an den Messkunden."""
    BESCHAFFUNGSRECHNUNG = "BESCHAFFUNGSRECHNUNG"
    """Rechnungen zwischen einem  Händler und Einkäufer von Energie."""
    AUSGLEICHSENERGIERECHNUNG = "AUSGLEICHSENERGIERECHNUNG"
    """Rechnung an den Verursacher von Ausgleichsenergie."""
    ABSCHLUSSRECHNUNG = "ABSCHLUSSRECHNUNG"  #: ABSCHLUSSRECHNUNG
    ABSCHLAGSRECHNUNG = "ABSCHLAGSRECHNUNG"  #: ABSCHLAGSRECHNUNG
    TURNUSRECHNUNG = "TURNUSRECHNUNG"  #: TURNUSRECHNUNG
    MONATSRECHNUNG = "MONATSRECHNUNG"  #: MONATSRECHNUNG
    ZWISCHENRECHNUNG = "ZWISCHENRECHNUNG"  #: ZWISCHENRECHNUNG
    INTEGRIERTE_13TE_RECHNUNG = "INTEGRIERTE_13TE_RECHNUNG"  #: INTEGRIERTE_13TE_RECHNUNG
    ZUSAETZLICHE_13TE_RECHNUNG = "ZUSAETZLICHE_13TE_RECHNUNG"  #: ZUSAETZLICHE_13TE_RECHNUNG



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Regionskriteriumtyp(StrEnum):
    """
    Klassifizierung der Kriterien für eine regionale Eingrenzung.
    """

    BUNDESLANDKENNZIFFER = "BUNDESLANDKENNZIFFER"
    """offizielle Bundeslandkennziffer"""
    BUNDESLAND_NAME = "BUNDESLAND_NAME"
    """Bundesland Name"""
    MARKTGEBIET_NUMMER = "MARKTGEBIET_NUMMER"
    """offizielle Marktgebiet-Codenummer"""
    MARKTGEBIET_NAME = "MARKTGEBIET_NAME"
    """Marktgebiet Name"""
    REGELGEBIET_NUMMER = "REGELGEBIET_NUMMER"
    """offizielle Regelgebiet Nummer"""
    REGELGEBIET_NAME = "REGELGEBIET_NAME"
    """Regelgebiet Name"""
    NETZ_STROM = "NETZ_STROM"
    """Identifikation des Netzes Strom"""
    NETZ_GAS = "NETZ_GAS"
    """Identifikation des Netzes Gas"""
    NETZBETREIBER_NUMMER_STROM = "NETZBETREIBER_NUMMER_STROM"
    """offizielle Netzbetreiber-Codenummer Strom"""
    NETZBETREIBER_NUMMER_GAS = "NETZBETREIBER_NUMMER_GAS"
    """offizielle Netzbetreiber-Codenummer Gas"""
    NETZBETREIBER_NAME_STROM = "NETZBETREIBER_NAME_STROM"
    """Netzbetreiber Name Strom"""
    NETZBETREIBER_NAME_GAS = "NETZBETREIBER_NAME_GAS"
    """Netzbetreiber Name Gas"""
    BILANZIERUNGS_GEBIET_NUMMER = "BILANZIERUNGS_GEBIET_NUMMER"
    """Strom: Bilanzierungsgebietsnummer, Gas: Netzkontonummer"""
    MSB_NUMMER = "MSB_NUMMER"
    """offizielle Messstellenbetreiber-Codenummer"""
    MSB_NAME = "MSB_NAME"
    """Name des MSB"""
    VERSORGER_NUMMER = "VERSORGER_NUMMER"
    """offizielle Lieferanten-Codenummer eines Versorgers"""
    VERSORGER_NAME = "VERSORGER_NAME"
    """Name eines Versorgers"""
    GRUNDVERSORGER_NUMMER_STROM = "GRUNDVERSORGER_NUMMER_STROM"
    """offizielle Lieferanten-Codenummer des Strom-Grundversorgers"""
    GRUNDVERSORGER_NAME_STROM = "GRUNDVERSORGER_NAME_STROM"
    """Name des Strom-Grundversorgers"""
    GRUNDVERSORGER_NUMMER_GAS = "GRUNDVERSORGER_NUMMER_GAS"
    """offizielle Lieferanten-Codenummer des Gas-Grundversorgers"""
    GRUNDVERSORGER_NAME_GAS = "GRUNDVERSORGER_NAME_GAS"
    """Name des Gas-Grundversorgers"""
    KREIS_NAME = "KREIS_NAME"
    """Kreis"""
    KREISKENNZIFFER = "KREISKENNZIFFER"
    """offizielle Kreiskennziffer"""
    GEMEINDE_NAME = "GEMEINDE_NAME"
    """Gemeinde"""
    GEMEINDEKENNZIFFER = "GEMEINDEKENNZIFFER"
    """offizielle Gemeindekennziffer"""
    POSTLEITZAHL = "POSTLEITZAHL"
    """Postleitzahl"""
    ORT = "ORT"
    """Ort"""
    POSTORT = "POSTORT"
    """Kriterium bestehend aus Postleitzahl und Ort"""
    EINWOHNERZAHL_GEMEINDE = "EINWOHNERZAHL_GEMEINDE"
    """Einwohnerzahl Gemeinde"""
    EINWOHNERZAHL_ORT = "EINWOHNERZAHL_ORT"
    """Einwohnerzahl Ort"""
    KM_UMKREIS = "KM_UMKREIS"
    """km Umkreis"""
    BUNDESWEIT = "BUNDESWEIT"
    """Bundesweite Betrachtung"""
    PLZ_BEREICH = "PLZ_BEREICH"
    """Postleitzahlenbereich"""



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Tariftyp(StrEnum):
    """
    Zur Differenzierung von Grund/Ersatzversorgungstarifen und sonstigen angebotenen Tarifen.
    """

    GRUND_ERSATZVERSORGUNG = "GRUND_ERSATZVERSORGUNG"
    """Grund- und Ersatzversorgung"""
    GRUNDVERSORGUNG = "GRUNDVERSORGUNG"
    """Grundversorgung"""
    ERSATZVERSORGUNG = "ERSATZVERSORGUNG"
    """Ersatzversorgung"""
    SONDERTARIF = "SONDERTARIF"
    """Sondertarif"""



"""
Contains Enums for Fallgruppenzuordnung
"""

from bo4e.enum.strenum import StrEnum


class Fallgruppenzuordnung(StrEnum):
    """
    Fallgruppenzuordnung nach edi@energy
    """

    GABI_RLM_MIT_TAGESBAND = "GABI_RLM_MIT_TAGESBAND"
    """RLM mit Tagesband"""
    GABI_RLM_OHNE_TAGESBAND = "GABI_RLM_OHNE_TAGESBAND"
    """RLM ohne Tagesband"""
    GABI_RLM_IM_NOMINIERUNGSERSATZVERFAHREN = "GABI_RLM_IM_NOMINIERUNGSERSATZVERFAHREN"
    """RLM im Nominierungsersatzverfahren"""



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Lokationstyp(StrEnum):
    """
    Gibt an, ob es sich um eine Markt- oder Messlokation handelt.
    """

    MALO = "MALO"
    """Marktlokation"""
    MELO = "MELO"
    """Messlokation"""
    NELO = "NELO"
    """Netzlokation"""
    SR = "SR"
    """Steuerbare Ressource"""
    TR = "TR"
    """Technische Ressource"""



"""
Contains RegionalePreisgarantie class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .preisgarantie import Preisgarantie

if TYPE_CHECKING:
    from .regionalegueltigkeit import RegionaleGueltigkeit

# pylint: disable=too-few-public-methods


@postprocess_docstring
class RegionalePreisgarantie(Preisgarantie):
    """
    Abbildung einer Preisgarantie mit regionaler Abgrenzung

    .. raw:: html

        <object data="../_static/images/bo4e/com/RegionalePreisgarantie.svg" type="image/svg+xml"></object>

    .. HINT::
        `RegionalePreisgarantie JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/RegionalePreisgarantie.json>`_

    """

    typ: Annotated[Literal[ComTyp.REGIONALEPREISGARANTIE], Field(alias="_typ")] = (
        ComTyp.REGIONALEPREISGARANTIE  # type:ignore[assignment]
    )

    regionale_gueltigkeit: Optional["RegionaleGueltigkeit"] = None
    """Regionale Eingrenzung der Preisgarantie."""



"""
Contains Sigmoidparameter class
"""

from decimal import Decimal
from typing import Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

# pylint:disable=invalid-name, too-few-public-methods


@postprocess_docstring
class Sigmoidparameter(COM):
    """
    Die Sigmoid-Funktion, beispielsweise zur Berechnung eines Leistungspreises hat die Form:
    LP=A/(1+(P/B)^C)+D

    .. raw:: html

        <object data="../_static/images/bo4e/com/Sigmoidparameter.svg" type="image/svg+xml"></object>

    .. HINT::
        `Sigmoidparameter JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Sigmoidparameter.json>`_

    """

    typ: Annotated[Literal[ComTyp.SIGMOIDPARAMETER], Field(alias="_typ")] = ComTyp.SIGMOIDPARAMETER

    A: Optional[Decimal] = None
    """Briefmarke Ortsverteilnetz (EUR/kWh)"""
    B: Optional[Decimal] = None
    """Wendepunkt für die bepreiste Menge (kW)"""
    C: Optional[Decimal] = None
    """Exponent (einheitenlos)"""
    D: Optional[Decimal] = None
    """Briefmarke Transportnetz (EUR/kWh)"""



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class AufAbschlagsziel(StrEnum):
    """
    Der Preis, auf den sich ein Auf- oder Abschlag bezieht.
    """

    ARBEITSPREIS_EINTARIF = "ARBEITSPREIS_EINTARIF"
    """Auf-/Abschlag auf den Arbeitspreis Eintarif"""
    ARBEITSPREIS_HT = "ARBEITSPREIS_HT"
    """Auf-/Abschlag auf den Arbeitspreis HT"""
    ARBEITSPREIS_NT = "ARBEITSPREIS_NT"
    """Auf-/Abschlag auf den Arbeitspreis NT"""
    ARBEITSPREIS_HT_NT = "ARBEITSPREIS_HT_NT"
    """Auf-/Abschlag auf den Arbeitspreis HT und NT"""
    GRUNDPREIS = "GRUNDPREIS"
    """Auf-/Abschlag auf den Grundpreis"""
    GESAMTPREIS = "GESAMTPREIS"
    """Auf-/Abschlag auf den Gesamtpreis"""



"""
Contains RegionaleGueltigkeit class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.gueltigkeitstyp import Gueltigkeitstyp
    from .kriteriumwert import KriteriumWert

# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module


@postprocess_docstring
class RegionaleGueltigkeit(COM):
    """
    Mit dieser Komponente können regionale Gültigkeiten, z.B. für Tarife, Zu- und Abschläge und Preise definiert werden.

    .. raw:: html

        <object data="../_static/images/bo4e/com/RegionaleGueltigkeit.svg" type="image/svg+xml"></object>

    .. HINT::
        `RegionaleGueltigkeit JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/RegionaleGueltigkeit.json>`_

    """

    typ: Annotated[Literal[ComTyp.REGIONALEGUELTIGKEIT], Field(alias="_typ")] = ComTyp.REGIONALEGUELTIGKEIT

    gueltigkeitstyp: Optional["Gueltigkeitstyp"] = None
    """Unterscheidung ob Positivliste oder Negativliste übertragen wird"""
    kriteriums_werte: Optional[list["KriteriumWert"]] = None
    """Hier stehen die Kriterien, die die regionale Gültigkeit festlegen"""



"""
Contains Lastgang class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module
from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

if TYPE_CHECKING:
    from ..bo.marktlokation import Marktlokation
    from ..bo.messlokation import Messlokation
    from ..com.menge import Menge
    from ..com.zeitreihenwert import Zeitreihenwert
    from ..enum.mengeneinheit import Mengeneinheit
    from ..enum.sparte import Sparte


# pylint: disable=too-many-instance-attributes, too-few-public-methods


@postprocess_docstring
class Lastgang(Geschaeftsobjekt):
    """
    Modell zur Abbildung eines Lastganges;
    In diesem Modell werden die Messwerte mit einem vollständigen Zeitintervall (zeit_intervall_laenge) angegeben und es bietet daher eine hohe Flexibilität in der Übertragung jeglicher zeitlich veränderlicher Messgrössen.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Lastgang.svg" type="image/svg+xml"></object>

    .. HINT::
        `Lastgang JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Lastgang.json>`_

    """

    typ: Annotated[Literal[BoTyp.LASTGANG], Field(alias="_typ")] = BoTyp.LASTGANG
    sparte: Optional["Sparte"] = None
    """Angabe, ob es sich um einen Gas- oder Stromlastgang handelt"""
    messgroesse: Optional["Mengeneinheit"] = None
    """Definition der gemessenen Größe anhand ihrer Einheit"""
    #:Marktlokation, zu der der Lastgang gehört
    marktlokation: Optional["Marktlokation"] = None
    #:Marktlokation, zu der der Lastgang gehört
    messlokation: Optional["Messlokation"] = None
    werte: Optional[list["Zeitreihenwert"]] = None
    """Die im Lastgang enthaltenen Messwerte"""
    version: Optional[str] = None
    """Versionsnummer des Lastgangs"""
    obis_kennzahl: Optional[str] = None
    """Die OBIS-Kennzahl für den Wert, die festlegt, welche Größe mit dem Stand gemeldet wird, z.B. '1-0:1.8.1'"""
    zeit_intervall_laenge: Optional["Menge"]



import pytest

from bo4e.com.tagesparameter import Tagesparameter
from tests.serialization_helper import assert_serialization_roundtrip


class TestTagesparameter:
    @pytest.mark.parametrize(
        "tagesparameter",
        [
            pytest.param(
                Tagesparameter(
                    klimazone="7624q",
                    temperaturmessstelle="1234x",
                    dienstanbieter="ZT1",
                    herausgeber="BDEW",
                )
            ),
        ],
    )
    def test_serialization_roundtrip(self, tagesparameter: Tagesparameter) -> None:
        """
        Test de-/serialisation of Tagesparameter.
        """
        assert_serialization_roundtrip(tagesparameter)



"""
Contains Marktgebietinfo class
"""

from typing import Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

# pylint: disable=too-few-public-methods


@postprocess_docstring
class MarktgebietInfo(COM):
    """
    Informationen zum Marktgebiet im Gas.

    .. raw:: html

        <object data="../_static/images/bo4e/com/MarktgebietInfo.svg" type="image/svg+xml"></object>

    .. HINT::
        `MarktgebietInfo JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/MarktgebietInfo.json>`_

    """

    typ: Annotated[Literal[ComTyp.MARKTGEBIETINFO], Field(alias="_typ")] = ComTyp.MARKTGEBIETINFO

    marktgebiet: Optional[str] = None
    """Der Name des Marktgebietes"""
    marktgebietcode: Optional[str] = None
    """Die standardisierte Codenummer des Marktgebietes"""



"""
Contains Angebotsposition class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:

    from .betrag import Betrag
    from .menge import Menge
    from .preis import Preis

# pylint: disable=too-few-public-methods


@postprocess_docstring
class Angebotsposition(COM):
    """
    Unterhalb von Angebotsteilen sind die Angebotspositionen eingebunden.
    Hier werden die angebotenen Bestandteile einzeln aufgeführt. Beispiel:
    Positionsmenge: 4000 kWh
    Positionspreis: 24,56 ct/kWh
    Positionskosten: 982,40 EUR

    .. raw:: html

        <object data="../_static/images/bo4e/com/Angebotsposition.svg" type="image/svg+xml"></object>

    .. HINT::
        `Angebotsposition JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Angebotsposition.json>`_

    """

    typ: Annotated[Literal[ComTyp.ANGEBOTSPOSITION], Field(alias="_typ")] = ComTyp.ANGEBOTSPOSITION

    positionsbezeichnung: Optional[str] = None
    """Bezeichnung der jeweiligen Position des Angebotsteils"""
    positionspreis: Optional["Preis"] = None
    """Preis pro Einheit/Stückpreis des angebotenen Artikels."""

    positionsmenge: Optional["Menge"] = None
    """Menge des angebotenen Artikels (z.B. Wirkarbeit in kWh), in dieser Angebotsposition"""
    positionskosten: Optional["Betrag"] = None
    """Kosten (positionspreis * positionsmenge) für diese Angebotsposition"""

    # for a preis = menge*times validation we first need to resolve
    # https://github.com/Hochfrequenz/BO4E-python/issues/126



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Ausschreibungsportal(StrEnum):
    """
    Aufzählung der unterstützten Ausschreibungsportale.
    """

    ENPORTAL = "ENPORTAL"
    """enPORTAL"""
    ENERGIE_AGENTUR = "ENERGIE_AGENTUR"
    """EnergieAgentur.NRW"""
    BMWI = "BMWI"
    """BMWI-Ausschreibungen"""
    ENERGIE_HANDELSPLATZ = "ENERGIE_HANDELSPLATZ"
    """energie-handelsplatz.de"""
    BUND = "BUND"
    """BUND.DE"""
    VERA_ONLINE = "VERA_ONLINE"
    """vera_online.de"""
    ISPEX = "ISPEX"
    """ispex.de"""
    ENERGIEMARKTPLATZ = "ENERGIEMARKTPLATZ"
    """energiemarktplatz.de"""
    EVERGABE = "EVERGABE"
    """evergabe.de"""
    DTAD = "DTAD"
    """dtad.de"""



"""
Contains Geschaeftspartner class
"""

# pylint: disable=too-many-instance-attributes, too-few-public-methods, disable=duplicate-code
from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

if TYPE_CHECKING:
    from ..com.adresse import Adresse
    from ..com.kontaktweg import Kontaktweg
    from ..enum.anrede import Anrede
    from ..enum.geschaeftspartnerrolle import Geschaeftspartnerrolle
    from ..enum.organisationstyp import Organisationstyp
    from ..enum.titel import Titel
    from .person import Person


@postprocess_docstring
class Geschaeftspartner(Geschaeftsobjekt):
    """
    Mit diesem Objekt können Geschäftspartner übertragen werden.
    Sowohl Unternehmen, als auch Privatpersonen können Geschäftspartner sein.
    Hinweis: "Marktteilnehmer" haben ein eigenes BO, welches sich von diesem BO ableitet.
    Hier sollte daher keine Zuordnung zu Marktrollen erfolgen.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Geschaeftspartner.svg" type="image/svg+xml"></object>

    .. HINT::
        `Geschaeftspartner JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Geschaeftspartner.json>`_

    """

    typ: Annotated[Literal[BoTyp.GESCHAEFTSPARTNER], Field(alias="_typ")] = BoTyp.GESCHAEFTSPARTNER
    anrede: Optional["Anrede"] = None
    """Mögliche Anrede der Person"""
    individuelle_anrede: Optional[str] = None
    """
    Im Falle einer nicht standardisierten Anrede kann hier eine frei definierbare Anrede vorgegeben werden.
    Beispiel: "Vereinsgemeinschaft", "Pfarrer", "Hochwürdigster Herr Abt".
    """
    titel: Optional["Titel"] = None
    """Möglicher Titel der Person"""
    vorname: Optional[str] = None
    """Vorname der Person"""
    nachname: Optional[str] = None
    """Nachname (Familienname) der Person"""

    ansprechpartner: Optional[list["Person"]] = None
    organisationstyp: Optional["Organisationstyp"] = None
    """
    Kennzeichnung ob es sich um ein Gewerbe/Unternehmen, eine Privatperson oder eine andere Art von Organisation handelt.
    """
    organisationsname: Optional[str] = None
    """
    Name der Firma, wenn Gewerbe oder andere Organisation.
    """
    kontaktwege: Optional[list["Kontaktweg"]] = None
    """Kontaktwege des Geschäftspartners"""
    geschaeftspartnerrollen: Optional[list["Geschaeftspartnerrolle"]] = None
    """Rollen, die die Geschäftspartner inne haben (z.B. Interessent, Kunde)"""
    handelsregisternummer: Optional[str] = None
    """Handelsregisternummer des Geschäftspartners"""
    amtsgericht: Optional[str] = None
    """Amtsgericht bzw Handelsregistergericht, das die Handelsregisternummer herausgegeben hat"""
    umsatzsteuer_id: Optional[str] = None
    """
    Die Steuer-ID des Geschäftspartners; Beispiel: "DE 813281825"
    """
    glaeubiger_id: Optional[str] = None
    """
    Die Gläubiger-ID welche im Zahlungsverkehr verwendet wird; Z.B. "DE 47116789"
    """
    website: Optional[str] = None
    """Internetseite des Marktpartners"""
    adresse: Optional["Adresse"] = None
    """Adresse des Geschäftspartners"""
    # Todo: Add optional connection to marktteilnehmer as discussed in workshop
    # not clear what is the best solution here - circular import marktteilnehmer?
    # discussed in workshop on Feb 6 2024: yes we need the bidirectional option, let's figure out a solution somehow.



from decimal import Decimal
from typing import Any

import pytest
from pydantic import ValidationError

from bo4e import Preisstaffel, Sigmoidparameter
from tests.serialization_helper import assert_serialization_roundtrip


class TestPreisstaffel:
    @pytest.mark.parametrize(
        "preisstaffel",
        [
            pytest.param(
                Preisstaffel(
                    einheitspreis=Decimal(40.0),
                    staffelgrenze_von=Decimal(12.5),
                    staffelgrenze_bis=Decimal(25.0),
                    sigmoidparameter=Sigmoidparameter(),
                    artikel_id="4-5-6",
                ),
                id="all attributes",
            ),
        ],
    )
    def test_serialization_roundtrip(self, preisstaffel: Preisstaffel) -> None:
        """
        Test de-/serialisation of Preisstaffel.
        """
        assert_serialization_roundtrip(preisstaffel)

    @pytest.mark.parametrize(
        "not_a_sigmoid_parameter",
        [
            pytest.param(17),  # not a sigmoid parameter instance
            pytest.param("foo"),  # not a sigmoid parameter instance
        ],
    )
    def test_failing_validation(self, not_a_sigmoid_parameter: Any) -> None:
        with pytest.raises(ValidationError) as excinfo:
            _ = Preisstaffel(
                einheitspreis=Decimal(40.0),
                staffelgrenze_von=Decimal(12.5),
                staffelgrenze_bis=Decimal(25.0),
                sigmoidparameter=not_a_sigmoid_parameter,
                artikel_id="4-5-6",
            )

        assert "1 validation error" in str(excinfo.value)
        assert "sigmoidparameter" in str(excinfo.value)



"""
Contains class Ausschreibungsdetail
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.zaehlertyp import Zaehlertyp
    from .adresse import Adresse
    from .menge import Menge
    from .zeitraum import Zeitraum

# pylint: disable=too-few-public-methods, too-many-instance-attributes


@postprocess_docstring
class Ausschreibungsdetail(COM):
    """
    Die Komponente Ausschreibungsdetail wird verwendet um die Informationen zu einer Abnahmestelle innerhalb eines
    Ausschreibungsloses abzubilden.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Ausschreibungsdetail.svg" type="image/svg+xml"></object>

    .. HINT::
        `Ausschreibungsdetail JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Ausschreibungsdetail.json>`_

    """

    typ: Annotated[Literal[ComTyp.AUSSCHREIBUNGSDETAIL], Field(alias="_typ")] = ComTyp.AUSSCHREIBUNGSDETAIL

    marktlokations_id: Optional[str] = None
    """Identifikation einer ausgeschriebenen Marktlokation"""
    netzebene_lieferung: Optional[str] = None
    """In der angegebenen Netzebene wird die Marktlokation versorgt, z.B. MSP für Mittelspannung"""
    netzebene_messung: Optional[str] = None
    """In der angegebenen Netzebene wird die Lokation gemessen, z.B. NSP für Niederspannung"""
    marktlokationsadresse: Optional["Adresse"] = None
    """Die Adresse an der die Marktlokation sich befindet"""
    lieferzeitraum: Optional["Zeitraum"] = None
    """Angefragter Zeitraum für die ausgeschriebene Belieferung"""

    netzbetreiber: Optional[str] = None
    """Bezeichnung des zuständigen Netzbetreibers, z.B. 'Stromnetz Hamburg GmbH'"""
    kunde: Optional[str] = None
    """Bezeichnung des Kunden, der die Marktlokation nutzt"""
    zaehlernummer: Optional[str] = None
    """Die Bezeichnung des Zählers an der Marktlokation"""
    marktlokationsbezeichnung: Optional[str] = None
    """Bezeichnung für die Lokation, z.B. 'Zentraler Einkauf, Hamburg'"""

    zaehlertechnik: Optional["Zaehlertyp"] = None
    """Spezifikation, um welche Zählertechnik es sich im vorliegenden Fall handelt, z.B. Leistungsmessung"""
    ist_lastgang_vorhanden: Optional[bool] = None
    """
    Zeigt an, ob es zu der Marktlokation einen Lastgang gibt.
    Falls ja, kann dieser abgerufen werden und daraus die Verbrauchswerte ermittelt werden
    """

    prognose_jahresarbeit: Optional["Menge"] = None
    """Prognosewert für die Jahresarbeit der ausgeschriebenen Lokation"""
    prognose_arbeit_lieferzeitraum: Optional["Menge"] = None
    """Ein Prognosewert für die Arbeit innerhalb des angefragten Lieferzeitraums der ausgeschriebenen Lokation"""
    prognose_leistung: Optional["Menge"] = None
    """Prognosewert für die abgenommene maximale Leistung der ausgeschriebenen Lokation"""
    rechnungsadresse: Optional["Adresse"] = None
    """Die (evtl. abweichende) Rechnungsadresse"""



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Speicherart(StrEnum):
    """
    Art der Speicherung
    """

    WASSERSTOFFSPEICHER = "WASSERSTOFFSPEICHER"
    PUMPSPEICHER = "PUMPSPEICHER"
    BATTERIESPEICHER = "BATTERIESPEICHER"
    SONSTIGE_SPEICHERART = "SONSTIGE_SPEICHERART"



from decimal import Decimal

import pytest
from pydantic import ValidationError

from bo4e import Menge, Mengeneinheit
from tests.serialization_helper import assert_serialization_roundtrip


class TestMenge:
    @pytest.mark.parametrize(
        "menge",
        [
            pytest.param(
                Menge(wert=Decimal(3.41), einheit=Mengeneinheit.MWH),
            )
        ],
    )
    def test_serialization_roundtrip(self, menge: Menge) -> None:
        """
        Test de-/serialisation of Menge.
        """

        assert_serialization_roundtrip(menge)

    def test_wrong_datatype(self) -> None:
        """
        A string "3.14" would be casted to decimal from pydantic therefore no validation error would occure in this case.
        """
        with pytest.raises(ValidationError) as excinfo:
            _ = Menge(wert="hallo", einheit=Mengeneinheit.MWH)  # type: ignore[arg-type]

        assert "wert" in str(excinfo.value)



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Ausschreibungsstatus(StrEnum):
    """
    Bezeichnungen für die Ausschreibungsphasen
    """

    PHASE1 = "PHASE1"
    """Phase1: Teilnahmewettbewerb"""
    PHASE2 = "PHASE2"
    """Phase2: Angebotsphase"""
    PHASE3 = "PHASE3"
    """Phase3: Verhandlungsphase"""
    PHASE4 = "PHASE4"
    """Phase4: Zuschlagserteilung"""



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Waermenutzung(StrEnum):
    """
    Wärmenutzung Marktlokation
    """

    SPEICHERHEIZUNG = "SPEICHERHEIZUNG"
    """Speicherheizung"""
    WAERMEPUMPE = "WAERMEPUMPE"
    """Wärmepumpe"""
    DIREKTHEIZUNG = "DIREKTHEIZUNG"
    """Direktheizung"""



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Gebiettyp(StrEnum):
    """
    List of possible Gebiettypen.
    """

    REGELZONE = "REGELZONE"
    """Regelzone"""
    MARKTGEBIET = "MARKTGEBIET"
    """Marktgebiet"""
    BILANZIERUNGSGEBIET = "BILANZIERUNGSGEBIET"
    """Bilanzierungsgebiet"""
    VERTEILNETZ = "VERTEILNETZ"
    """Verteilnetz"""
    TRANSPORTNETZ = "TRANSPORTNETZ"
    """Transportnetz"""
    REGIONALNETZ = "REGIONALNETZ"
    """Regionalnetz"""
    AREALNETZ = "AREALNETZ"
    """Arealnetz"""
    GRUNDVERSORGUNGSGEBIET = "GRUNDVERSORGUNGSGEBIET"
    """Grundversorgungsgebiet"""
    VERSORGUNGSGEBIET = "VERSORGUNGSGEBIET"
    """Versorgungsgebiet"""



"""
Contains Kosten class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

if TYPE_CHECKING:
    from ..com.betrag import Betrag
    from ..com.kostenblock import Kostenblock
    from ..com.zeitraum import Zeitraum
    from ..enum.kostenklasse import Kostenklasse


# pylint: disable=too-many-instance-attributes, too-few-public-methods
# pylint: disable=no-name-in-module


@postprocess_docstring
class Kosten(Geschaeftsobjekt):
    """
    Dieses BO wird zur Übertagung von hierarchischen Kostenstrukturen verwendet.
    Die Kosten werden dabei in Kostenblöcke und diese wiederum in Kostenpositionen strukturiert.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Kosten.svg" type="image/svg+xml"></object>

    .. HINT::
        `Kosten JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Kosten.json>`_

    """

    typ: Annotated[Literal[BoTyp.KOSTEN], Field(alias="_typ")] = BoTyp.KOSTEN
    kostenklasse: Optional["Kostenklasse"] = None
    """Klasse der Kosten, beispielsweise Fremdkosten"""
    gueltigkeit: Optional["Zeitraum"] = None
    """Für diesen Zeitraum wurden die Kosten ermittelt"""
    kostenbloecke: Optional[list["Kostenblock"]] = None
    """In Kostenblöcken werden Kostenpositionen zusammengefasst. Beispiele: Netzkosten, Umlagen, Steuern etc"""

    summe_kosten: Optional[list["Betrag"]] = None
    """Die Gesamtsumme über alle Kostenblöcke und -positionen"""



from datetime import datetime, timezone

import pytest

from bo4e import Preisgarantie, Preisgarantietyp, Zeitraum
from tests.serialization_helper import assert_serialization_roundtrip


class TestPreisgarantie:
    @pytest.mark.parametrize(
        "preisgarantie",
        [
            pytest.param(
                Preisgarantie(
                    preisgarantietyp=Preisgarantietyp.NUR_ENERGIEPREIS,
                    zeitliche_gueltigkeit=Zeitraum(
                        startdatum=datetime(2020, 1, 1, tzinfo=timezone.utc),
                        enddatum=datetime(2020, 4, 1, tzinfo=timezone.utc),
                    ),
                ),
            ),
        ],
    )
    def test_preisgarantie_required_attributes(self, preisgarantie: Preisgarantie) -> None:
        """
        Test de-/serialisation of Preisgarantie with minimal attributes.
        """
        assert_serialization_roundtrip(preisgarantie)



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Marktrolle(StrEnum):
    """
    Diese Rollen kann ein Marktteilnehmer einnehmen.
    """

    BTR = "BTR"
    """Betreiber einer technischen Ressource"""
    BIKO = "BIKO"
    """Bilanzkoordinator / Marktgebietsverantwortlicher"""
    BKV = "BKV"
    """Bilanzkreisverantwortlicher"""
    DP = "DP"
    """Data Provider"""
    EIV = "EIV"
    """Einsatzverantwortlicher"""
    ESA = "ESA"
    """Energieserviceanbieter des Anschlussnutzers"""
    KN = "KN"
    """ Kapazitätsnutzer"""
    LF = "LF"
    """Lieferant"""
    MGV = "MGV"
    """Marktgebietsverantwortlicher"""
    MSB = "MSB"
    """Messstellenbetreiber"""
    NB = "NB"
    """Netzbetreiber"""
    RB = "RB"
    """Registerbetreiber"""
    UENB = "UENB"
    """Übertragungsnetzbetreiber"""



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class ZaehlertypSpezifikation(StrEnum):
    """
    Bei diesem Enum handelt es sich um die Abbildung von besonderen Zählertyp-Spezifikationen der Sparten Strom und Gas.
    """

    EDL40 = "EDL40"
    """EDL40"""
    EDL21 = "EDL21"
    """EDL21"""
    SONSTIGER_EHZ = "SONSTIGER_EHZ"
    """SONSTIGER_EHZ"""
    MME_STANDARD = "MME_STANDARD"
    """MME_STANDARD"""
    MME_MEDA = "MME_MEDA"
    """MME_MEDA"""



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


# pylint: disable=line-too-long
class Geraeteklasse(StrEnum):
    """
    Auflistung möglicher übergreifenden Geräteklassen.
    """

    WANDLER = "WANDLER"
    """Wandler"""
    KOMMUNIKATIONSEINRICHTUNG = "KOMMUNIKATIONSEINRICHTUNG"
    """Kommunikationseinrichtung"""
    TECHNISCHE_STEUEREINRICHTUNG = "TECHNISCHE_STEUEREINRICHTUNG"
    """Technische Steuereinrichtung"""
    MENGENUMWERTER = "MENGENUMWERTER"
    """Mengenumwerter"""
    SMARTMETER_GATEWAY = "SMARTMETER_GATEWAY"
    """Smartmeter-Gateway"""
    STEUERBOX = "STEUERBOX"
    """Steuerbox"""
    ZAEHLEINRICHTUNG = "ZAEHLEINRICHTUNG"
    """Zaehleinrichtung"""



"""
Contains Energieherkunft class
"""

from decimal import Decimal
from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.erzeugungsart import Erzeugungsart


# pylint: disable=no-name-in-module


# pylint: disable=too-few-public-methods


@postprocess_docstring
class Energieherkunft(COM):
    """
    Abbildung einer Energieherkunft

    .. raw:: html

        <object data="../_static/images/bo4e/com/Energieherkunft.svg" type="image/svg+xml"></object>

    .. HINT::
        `Energieherkunft JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Energieherkunft.json>`_

    """

    typ: Annotated[Literal[ComTyp.ENERGIEHERKUNFT], Field(alias="_typ")] = ComTyp.ENERGIEHERKUNFT

    erzeugungsart: Optional["Erzeugungsart"] = None
    """Art der Erzeugung der Energie."""
    anteil_prozent: Optional[Decimal] = None
    """Prozentualer Anteil der jeweiligen Erzeugungsart."""



"""
Contains PreisblattNetnutzung class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .preisblatt import Preisblatt

if TYPE_CHECKING:
    from ..enum.bilanzierungsmethode import Bilanzierungsmethode
    from ..enum.kundengruppe import Kundengruppe
    from ..enum.netzebene import Netzebene


# pylint: disable=too-few-public-methods


@postprocess_docstring
class PreisblattNetznutzung(Preisblatt):
    """
    Die Variante des Preisblattmodells zur Abbildung der Netznutzungspreise

    .. raw:: html

        <object data="../_static/images/bo4e/bo/PreisblattNetznutzung.svg" type="image/svg+xml"></object>

    .. HINT::
        `PreisblattNetznutzung JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/PreisblattNetznutzung.json>`_

    """

    typ: Annotated[Literal[BoTyp.PREISBLATTNETZNUTZUNG], Field(alias="_typ")] = (
        BoTyp.PREISBLATTNETZNUTZUNG  # type: ignore[assignment]
    )
    # required attributes (additional to those of Preisblatt)
    bilanzierungsmethode: Optional["Bilanzierungsmethode"] = None
    """Die Preise gelten für Marktlokationen der angebebenen Bilanzierungsmethode"""
    netzebene: Optional["Netzebene"] = None
    """Die Preise gelten für Marktlokationen in der angebebenen Netzebene"""
    kundengruppe: Optional["Kundengruppe"] = None

    # there are no optional attributes (additionally to those of Preisblatt)



"""
Contains class Lastprofil
"""

from typing import Annotated, Literal, Optional

from pydantic import Field

from bo4e.com.com import COM
from bo4e.com.tagesparameter import Tagesparameter
from bo4e.enum.comtyp import ComTyp
from bo4e.enum.profilart import Profilart
from bo4e.enum.profilverfahren import Profilverfahren


class Lastprofil(COM):
    """
    Informationen zum Lastprofil.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Lastprofil.svg" type="image/svg+xml"></object>

    .. HINT::
        `Lastprofil JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/Hochfrequenz/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Lastprofil.json>`_

    """

    typ: Annotated[Literal[ComTyp.LASTPROFIL], Field(alias="_typ")] = ComTyp.LASTPROFIL

    bezeichnung: Optional[str] = None  #: Bezeichnung des Profils, durch DVGW bzw. den Netzbetreiber vergeben (z.B. H0)
    profilschar: Optional[str] = (
        None  #: Bezeichnung der Profilschar, durch DVGW bzw. den Netzbetreiber vergeben (z.B. H0)
    )
    verfahren: Optional[Profilverfahren] = None  #: Verfahren des Profils (analytisch oder synthetisch)
    ist_einspeisung: Optional[bool] = None  #: Einspeiseprofil: True/False
    tagesparameter: Optional[Tagesparameter] = None  #: Klimazone / Temperaturmessstelle
    profilart: Optional[Profilart] = None  #: Profilart des Lastprofils, e.g. ART_STANDARDLASTPROFIL
    herausgeber: Optional[str] = None  #: Herausgeber des Lastprofil-Codes, e.g. BDEW



# pylint: disable=missing-module-docstring, line-too-long
from bo4e.enum.strenum import StrEnum


class Voraussetzungen(StrEnum):
    """
    Voraussetzungen, die erfüllt sein müssen, damit dieser Tarif zur Anwendung kommen kann.
    """

    EINZUGSERMAECHTIGUNG = "EINZUGSERMAECHTIGUNG"
    """Einzugsermaechtigung"""
    ZEITPUNKT = "ZEITPUNKT"
    """Vertrag muss zu einem bestimmten Zeitpunkt noch bestehen"""
    LIEFERANBINDUNG_EINE = "LIEFERANBINDUNG_EINE"
    """Lieferantenbindung für diese Energieart"""
    LIEFERANBINDUNG_ALLE = "LIEFERANBINDUNG_ALLE"
    """Lieferantenbindung für alle Energiearten, die der Versorger anbietet"""
    GEWERBE = "GEWERBE"
    """Gewerbenachweis"""
    LASTPROFIL = "LASTPROFIL"
    """Kunde muss einem bestimmten Lastprofil zuzuordnen sein"""
    ZAEHLERTYP_GROESSE = "ZAEHLERTYP_GROESSE"
    """bestimmter Zaehlertyp oder Zaehlergroeße"""
    AUSSCHLUSS_GROSSVERBRAUCHER = "AUSSCHLUSS_GROSSVERBRAUCHER"
    """Ausschluss von Großverbrauchern wie Industriekunden oder produzierendes Gewerbe"""
    NEUKUNDE = "NEUKUNDE"
    """Neukunden ohne bisherige Lieferanbindung"""
    BESTIMMTE_VERTRAGSFORMALITAETEN = "BESTIMMTE_VERTRAGSFORMALITAETEN"
    """bestimmte Vertragsformalitaeten wie z.B. Anmeldeformular muss an bestimmte Adresse versandt werden"""
    SELBSTABLESUNG = "SELBSTABLESUNG"
    """Selbstablesung durch den Kunden"""
    ONLINEVORAUSSETZUNG = "ONLINEVORAUSSETZUNG"
    """Onlinevoraussetzung"""
    MINDESTUMSATZ = "MINDESTUMSATZ"
    """Mindestumsatz in einer bestimmten Zeiteinheit wie z.B. Mindestjahresumsatz 2500 EURO"""
    ZUSATZPRODUKT = "ZUSATZPRODUKT"
    """Zusatzprodukt zu bereits bestehendem Lieferverhaeltnis beim Versorger in dieser Energieart"""
    NEUKUNDE_MIT_VORAUSSETZUNGEN = "NEUKUNDE_MIT_VORAUSSETZUNGEN"
    """geworbener Neukunde muss bestimmte Voraussetzungen erfüllen"""
    DIREKTVERTRIEB = "DIREKTVERTRIEB"
    """Kunde wird durch Direktvertrieb gewonnen"""
    ANSCHLUSSART = "ANSCHLUSSART"
    """Anlage mit bestimmter Anschlussart wie z.B. Festanschluss"""
    ANSCHLUSSWERT = "ANSCHLUSSWERT"
    """bestimmte Anschlusswerte wie z. B. mindestens 30 kW"""
    ALTER_KUNDENANLAGE = "ALTER_KUNDENANLAGE"
    """Alter einer Kundenanlage (z.B. Anlage wurde nach dem 01.01.2005 installiert)"""
    ANLAGEBESCHAFFENHEIT = "ANLAGEBESCHAFFENHEIT"
    """bestimmte Anlagebeschaffenheit, wie bivalente Energieversorgung, Geräte o.ä."""
    BETRIEBSSTUNDENBEGRENZUNG = "BETRIEBSSTUNDENBEGRENZUNG"
    """Betriebsstundenbegrenzung z.B. max 1500h/a oder mindestens 1000h/a"""
    FREIGABEZEITEN = "FREIGABEZEITEN"
    """vorgeschriebene Freigabezeiten"""
    FAMILIENSTRUKTUR = "FAMILIENSTRUKTUR"
    """Familienstruktur wie z.B. bestimmte Anzahl Kinder oder Personen im Haushalt oder Eheleute"""
    MITGLIEDSCHAFT = "MITGLIEDSCHAFT"
    """Mitgliedschaft in bestimmten Vereinen oder Verbaenden"""
    STAATLICHE_FOERDERUNG = "STAATLICHE_FOERDERUNG"
    """staatliche Foerderung wie z.B. Sozialtarif oder Studentenausweis"""
    BESONDERE_VERBRAUCHSSTELLE = "BESONDERE_VERBRAUCHSSTELLE"
    """besondere Verbrauchsstellen wie Kirchen, Vereinsgebaeude usw."""
    NIEDRIGENERGIE = "NIEDRIGENERGIE"
    """Niedrigenergieaustattung des Hauses"""
    ORTSTEILE_LIEFERGEBIET = "ORTSTEILE_LIEFERGEBIET"
    """nur für bestimmte Ortsteile in diesem Liefergebiet"""
    WAERMEBEDARF_ERDGAS = "WAERMEBEDARF_ERDGAS"
    """Wärmebedarf wird nur oder überwiegend mit Erdgas gedeckt"""
    MAX_ZAEHLER_LIEFERSTELLEN = "MAX_ZAEHLER_LIEFERSTELLEN"
    """beschränkt auf max. Anzahl Zähler oder Abnahmestellen"""
    LIEFERUNGSBESCHRAENKUNG_GASART = "LIEFERUNGSBESCHRAENKUNG_GASART"
    """Lieferungsbeschraenkung auf bestimmte Gasart, wie H-Gas oder L-Gas"""
    KOMBI_BONI = "KOMBI_BONI"
    """Kombination von Boni, von denen mindestens einer sehr unwahrscheinlich zu erreichen ist"""
    ALTVERTRAG = "ALTVERTRAG"
    """nur für Altvertraege, die weiterhin gueltig sind"""
    VORGESCHRIEBENE_ZUSATZANLAGE = "VORGESCHRIEBENE_ZUSATZANLAGE"
    """vorgeschriebene Zusatzanlage wie z.B. Solaranlage etc."""
    MEHRERE_ZAEHLER_ABNAHMESTELLEN = "MEHRERE_ZAEHLER_ABNAHMESTELLEN"
    """mehr als 1 Zaehler oder 1 Abnahmestelle"""
    BESTIMMTER_ABNAHMEFALL = "BESTIMMTER_ABNAHMEFALL"
    """bestimmter Abnahmefall wie z.B. nur Gemeinschaftsheizungen o.ae."""
    ZUSATZMODALITAET = "ZUSATZMODALITAET"
    """Zahlungsmodalitaet wie z.B. halbjaehrliche Zahlungsweise"""
    NACHWEIS_ZAHLUNGSFAEHIGKEIT = "NACHWEIS_ZAHLUNGSFAEHIGKEIT"
    """Nachweis der Zahlungsfaehigkeit wie z.B. Bonitaetsprüfung"""
    UMSTELLUNG_ENERGIEART = "UMSTELLUNG_ENERGIEART"
    """Umstellung der Energieart"""



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Anrede(StrEnum):
    """
    Übersicht möglicher Anreden, z.B. eines Geschäftspartners.
    """

    HERR = "HERR"
    """"Herr"""
    FRAU = "FRAU"
    """Frau"""
    EHELEUTE = "EHELEUTE"
    """Eheleute"""
    FIRMA = "FIRMA"
    """Firma"""
    FAMILIE = "FAMILIE"
    """Familie"""
    ERBENGEMEINSCHAFT = "ERBENGEMEINSCHAFT"
    """Erbengemeinschaft"""
    GRUNDSTUECKSGEMEINSCHAFT = "GRUNDSTUECKSGEMEINSCHAFT"
    """Grundstücksgemeinschaft"""



from datetime import datetime, timezone

import pytest
from _decimal import Decimal

from bo4e.bo.bilanzierung import Bilanzierung
from bo4e.com.lastprofil import Lastprofil
from bo4e.com.menge import Menge
from bo4e.com.tagesparameter import Tagesparameter
from bo4e.enum.aggregationsverantwortung import Aggregationsverantwortung
from bo4e.enum.fallgruppenzuordnung import Fallgruppenzuordnung
from bo4e.enum.mengeneinheit import Mengeneinheit
from bo4e.enum.profilart import Profilart
from bo4e.enum.profiltyp import Profiltyp
from bo4e.enum.profilverfahren import Profilverfahren
from bo4e.enum.prognosegrundlage import Prognosegrundlage
from bo4e.enum.wahlrechtprognosegrundlage import WahlrechtPrognosegrundlage
from bo4e.enum.zeitreihentyp import Zeitreihentyp
from tests.serialization_helper import assert_serialization_roundtrip


class TestBilanzierung:
    @pytest.mark.parametrize(
        "bilanzierung",
        [
            pytest.param(
                Bilanzierung(
                    marktlokations_id="51238696781",
                    lastprofil=[
                        Lastprofil(
                            bezeichnung="foo",
                            profilschar="foo2",
                            verfahren=Profilverfahren.SYNTHETISCH,
                            ist_einspeisung=True,
                            tagesparameter=Tagesparameter(
                                klimazone="7624q",
                                temperaturmessstelle="1234x",
                                dienstanbieter="ZT1",
                                herausgeber="BDEW",
                            ),
                            profilart=Profilart.ART_LASTPROFIL,
                            herausgeber="BDEW",
                        )
                    ],
                    bilanzierungsbeginn=datetime(2022, 1, 1, 0, 0, 0, tzinfo=timezone.utc),
                    bilanzierungsende=datetime(2023, 1, 1, 0, 0, 0, tzinfo=timezone.utc),
                    bilanzkreis="foo",
                    jahresverbrauchsprognose=Menge(wert=Decimal(3.41), einheit=Mengeneinheit.MWH),
                    temperatur_arbeit=Menge(wert=Decimal(3.41), einheit=Mengeneinheit.MWH),
                    # todo: check einheiten
                    kundenwert=Menge(wert=Decimal(3.41), einheit=Mengeneinheit.MWH),
                    verbrauchsaufteilung=Decimal(1.5),
                    zeitreihentyp=Zeitreihentyp.EGS,
                    aggregationsverantwortung=Aggregationsverantwortung.VNB,
                    prognosegrundlage=Prognosegrundlage.WERTE,
                    details_prognosegrundlage=[Profiltyp.SLP_SEP],
                    wahlrecht_prognosegrundlage=WahlrechtPrognosegrundlage.DURCH_LF,
                    fallgruppenzuordnung=Fallgruppenzuordnung.GABI_RLM_MIT_TAGESBAND,
                    prioritaet=1,
                    grund_wahlrecht_prognosegrundlage=WahlrechtPrognosegrundlage.DURCH_LF_NICHT_GEGEBEN,
                )
            ),
        ],
    )
    def test_serialization_roundtrip(self, bilanzierung: Bilanzierung) -> None:
        assert_serialization_roundtrip(bilanzierung)



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Erzeugungsart(StrEnum):
    """
    Auflistung der Erzeugungsarten von Energie.
    """

    FOSSIL = "FOSSIL"
    """Fossile Brennstoffe"""
    KWK = "KWK"
    """Kraft-Waerme-Koppelung"""
    WIND = "WIND"
    """Windkraft"""
    SOLAR = "SOLAR"
    """Solarenergie"""
    KERNKRAFT = "KERNKRAFT"
    """Kernkraft"""
    WASSER = "WASSER"
    """Wasserkraft"""
    GEOTHERMIE = "GEOTHERMIE"
    """Geothermie"""
    BIOMASSE = "BIOMASSE"
    """Biomasse"""
    KOHLE = "KOHLE"
    """Kohle"""
    GAS = "GAS"
    """Erdgas"""
    SONSTIGE = "SONSTIGE"
    """Sonstige"""
    SONSTIGE_EEG = "SONSTIGE_EEG"
    """Sonstige nach EEG"""
    BIOGAS = "BIOGAS"
    """Biogas"""
    KLIMANEUTRALES_GAS = "KLIMANEUTRALES_GAS"
    """Klimaneutrales Erdgas"""



# pylint:disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class ComTyp(StrEnum):
    """
    Auflistung sämtlicher existierender Komponenten.

    .. HINT::
        Die Unterscheidung zwischen Komponenten und Geschäftsobjekten ist, dass Komponenten immer in einem Geschäftsobjekt enthalten sind.
        Komponenten sind also immer Teil von einem Geschäftsobjekt und können nicht alleine existieren.
    """

    ADRESSE = "ADRESSE"
    ANGEBOTSPOSITION = "ANGEBOTSPOSITION"
    ANGEBOTSTEIL = "ANGEBOTSTEIL"
    ANGEBOTSVARIANTE = "ANGEBOTSVARIANTE"
    AUFABSCHLAG = "AUFABSCHLAG"
    AUFABSCHLAGPROORT = "AUFABSCHLAGPROORT"
    AUFABSCHLAGREGIONAL = "AUFABSCHLAGREGIONAL"
    AUFABSCHLAGSTAFFELPROORT = "AUFABSCHLAGSTAFFELPROORT"
    AUSSCHREIBUNGSDETAIL = "AUSSCHREIBUNGSDETAIL"
    AUSSCHREIBUNGSLOS = "AUSSCHREIBUNGSLOS"
    BETRAG = "BETRAG"
    DIENSTLEISTUNG = "DIENSTLEISTUNG"
    ENERGIEHERKUNFT = "ENERGIEHERKUNFT"
    ENERGIEMIX = "ENERGIEMIX"
    FREMDKOSTENBLOCK = "FREMDKOSTENBLOCK"
    FREMDKOSTENPOSITION = "FREMDKOSTENPOSITION"
    GEOKOORDINATEN = "GEOKOORDINATEN"
    KATASTERADRESSE = "KATASTERADRESSE"
    KONFIGURATIONSPRODUKT = "KONFIGURATIONSPRODUKT"
    KONTAKTWEG = "KONTAKTWEG"
    KONZESSIONSABGABE = "KONZESSIONSABGABE"
    KOSTENBLOCK = "KOSTENBLOCK"
    KOSTENPOSITION = "KOSTENPOSITION"
    KRITERIUMWERT = "KRITERIUMWERT"
    LASTPROFIL = "LASTPROFIL"
    MARKTGEBIETINFO = "MARKTGEBIETINFO"
    MENGE = "MENGE"
    POSITIONSAUFABSCHLAG = "POSITIONSAUFABSCHLAG"
    PREIS = "PREIS"
    PREISGARANTIE = "PREISGARANTIE"
    PREISPOSITION = "PREISPOSITION"
    PREISSTAFFEL = "PREISSTAFFEL"
    RECHNUNGSPOSITION = "RECHNUNGSPOSITION"
    REGIONALEGUELTIGKEIT = "REGIONALEGUELTIGKEIT"
    REGIONALEPREISGARANTIE = "REGIONALEPREISGARANTIE"
    REGIONALEPREISSTAFFEL = "REGIONALEPREISSTAFFEL"
    REGIONALERAUFABSCHLAG = "REGIONALERAUFABSCHLAG"
    REGIONALETARIFPREISPOSITION = "REGIONALETARIFPREISPOSITION"
    REGIONSKRITERIUM = "REGIONSKRITERIUM"
    SIGMOIDPARAMETER = "SIGMOIDPARAMETER"
    STANDORTEIGENSCHAFTENGAS = "STANDORTEIGENSCHAFTENGAS"
    STANDORTEIGENSCHAFTENSTROM = "STANDORTEIGENSCHAFTENSTROM"
    STEUERBETRAG = "STEUERBETRAG"
    TAGESPARAMETER = "TAGESPARAMETER"
    TARIFBERECHNUNGSPARAMETER = "TARIFBERECHNUNGSPARAMETER"
    TARIFEINSCHRAENKUNG = "TARIFEINSCHRAENKUNG"
    TARIFPREIS = "TARIFPREIS"
    TARIFPREISPOSITION = "TARIFPREISPOSITION"
    TARIFPREISPOSITIONPROORT = "TARIFPREISPOSITIONPROORT"
    TARIFPREISSTAFFELPROORT = "TARIFPREISSTAFFELPROORT"
    UNTERSCHRIFT = "UNTERSCHRIFT"
    VERBRAUCH = "VERBRAUCH"
    VERTRAGSKONDITIONEN = "VERTRAGSKONDITIONEN"
    VERTRAGSTEIL = "VERTRAGSTEIL"
    VERWENDUNGSZWECKPROMARKTROLLE = "VERWENDUNGSZWECKPROMARKTROLLE"
    ZAEHLWERK = "ZAEHLWERK"
    ZAEHLZEITREGISTER = "ZAEHLZEITREGISTER"
    ZEITRAUM = "ZEITRAUM"
    ZEITREIHENWERT = "ZEITREIHENWERT"
    ZUSTAENDIGKEIT = "ZUSTAENDIGKEIT"



"""
Contains Vertrag class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

import pydantic
from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

if TYPE_CHECKING:
    from ..com.unterschrift import Unterschrift
    from ..com.vertragskonditionen import Vertragskonditionen
    from ..com.vertragsteil import Vertragsteil
    from ..enum.sparte import Sparte
    from ..enum.vertragsart import Vertragsart
    from ..enum.vertragsstatus import Vertragsstatus
    from .geschaeftspartner import Geschaeftspartner

# pylint: disable=unused-argument
# pylint: disable=no-name-in-module

# pylint: disable=too-many-instance-attributes, too-few-public-methods


@postprocess_docstring
class Vertrag(Geschaeftsobjekt):
    """
    Modell für die Abbildung von Vertragsbeziehungen;
    Das Objekt dient dazu, alle Arten von Verträgen, die in der Energiewirtschaft Verwendung finden, abzubilden.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Vertrag.svg" type="image/svg+xml"></object>

    .. HINT::
        `Vertrag JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Vertrag.json>`_

    """

    typ: Annotated[Literal[BoTyp.VERTRAG], Field(alias="_typ")] = BoTyp.VERTRAG
    # pylint: disable=duplicate-code
    vertragsnummer: Optional[str] = None
    """Eine im Verwendungskontext eindeutige Nummer für den Vertrag"""
    vertragsart: Optional["Vertragsart"] = None
    """Hier ist festgelegt, um welche Art von Vertrag es sich handelt."""
    vertragsstatus: Optional["Vertragsstatus"] = None
    """Gibt den Status des Vertrags an"""
    sparte: Optional["Sparte"] = None
    """Unterscheidungsmöglichkeiten für die Sparte"""
    vertragsbeginn: Optional[pydantic.AwareDatetime] = None
    """Gibt an, wann der Vertrag beginnt (inklusiv)"""
    vertragsende: Optional[pydantic.AwareDatetime] = None
    """Gibt an, wann der Vertrag (voraussichtlich) endet oder beendet wurde (exklusiv)"""
    # todo: add von/bis validator
    vertragspartner1: Optional["Geschaeftspartner"] = None
    """
    Der "erstgenannte" Vertragspartner.
    In der Regel der Aussteller des Vertrags.
    Beispiel: "Vertrag zwischen Vertragspartner 1 ..."
    """
    vertragspartner2: Optional["Geschaeftspartner"] = None
    """
    Der "zweitgenannte" Vertragspartner.
    In der Regel der Empfänger des Vertrags.
    Beispiel "Vertrag zwischen Vertragspartner 1 und Vertragspartner 2".
    """
    vertragsteile: Optional[list["Vertragsteil"]] = None
    """
    Der Vertragsteil wird dazu verwendet, eine vertragliche Leistung in Bezug zu einer Lokation
    (Markt- oder Messlokation) festzulegen.
    """

    beschreibung: Optional[str] = None
    """Beschreibung zum Vertrag"""
    vertragskonditionen: Optional["Vertragskonditionen"] = None
    """Festlegungen zu Laufzeiten und Kündigungsfristen"""
    unterzeichnervp1: Optional[list["Unterschrift"]] = None
    """Unterzeichner des Vertragspartners 1"""
    unterzeichnervp2: Optional[list["Unterschrift"]] = None
    """Unterzeichner des Vertragspartners 2"""



"""
Contains steuerbare Ressource class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

if TYPE_CHECKING:
    from ..bo.lokationszuordnung import Lokationszuordnung
    from ..com.konfigurationsprodukt import Konfigurationsprodukt
    from ..enum.marktrolle import Marktrolle
    from ..enum.steuerkanalleistungsbeschreibung import SteuerkanalLeistungsbeschreibung

# pylint: disable=too-many-instance-attributes, too-few-public-methods


@postprocess_docstring
class SteuerbareRessource(Geschaeftsobjekt):
    """
    Object containing information about a steuerbare Ressource

    .. raw:: html

        <object data="../_static/images/bo4e/bo/SteuerbareRessource.svg" type="image/svg+xml"></object>

    .. HINT::
        `SteuerbareRessource JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/SteuerbareRessource.json>`_

    """

    typ: Annotated[Literal[BoTyp.STEUERBARERESSOURCE], Field(alias="_typ")] = BoTyp.STEUERBARERESSOURCE

    steuerbare_ressource_id: Optional[str] = None
    """Id der steuerbaren Ressource"""
    steuerkanal_leistungsbeschreibung: Optional["SteuerkanalLeistungsbeschreibung"] = None
    """Leistungsbeschreibung des Steuerkanals"""
    zugeordnete_msb_codenummer: Optional[str] = None
    """Angabe des Messstellenbetreibers, der der Steuerbaren Ressource zugeordnet ist."""
    konfigurationsprodukte: Optional[list["Konfigurationsprodukt"]] = None
    """Produkt-Daten der Steuerbaren Ressource"""
    eigenschaft_msb_lokation: Optional["Marktrolle"] = None
    """Eigenschaft des Messstellenbetreibers an der Lokation"""
    lokationszuordnungen: Optional[list["Lokationszuordnung"]] = None
    """Lokationszuordnung, um bspw. die zugehörigen Messlokationen anzugeben"""
    lokationsbuendel_objektcode: Optional[str] = None
    """Lokationsbuendel Code, der die Funktion dieses BOs an der Lokationsbuendelstruktur beschreibt."""



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Themengebiet(StrEnum):
    """
    Über dieses ENUM kann eine thematische Zuordnung, beispielsweise eines Ansprechpartners, vorgenommen werden.
    """

    ALLGEMEINER_INFORMATIONSAUSTAUSCH = "ALLGEMEINER_INFORMATIONSAUSTAUSCH"
    """Allgemeiner Informationsaustausch"""
    AN_UND_ABMELDUNG = "AN_UND_ABMELDUNG"
    """An- und Abmeldung"""
    ANSPRECHPARTNER_ALLGEMEIN = "ANSPRECHPARTNER_ALLGEMEIN"
    """Ansprechpartner Allgemein"""
    ANSPRECHPARTNER_BDEW_DVGW = "ANSPRECHPARTNER_BDEW_DVGW"
    """Ansprechpartner BDEW/DVGW"""
    ANSPRECHPARTNER_IT_TECHNIK = "ANSPRECHPARTNER_IT_TECHNIK"
    """Ansprechpartner IT/Technik"""
    BILANZIERUNG = "BILANZIERUNG"
    """Bilanzierung"""
    BILANZKREISKOORDINATOR = "BILANZKREISKOORDINATOR"
    """Bilanzkreiskoordinator"""
    BILANZKREISVERANTWORTLICHER = "BILANZKREISVERANTWORTLICHER"
    """Bilanzkreisverantwortlicher"""
    DATENFORMATE_ZERTIFIKATE_VERSCHLUESSELUNGEN = "DATENFORMATE_ZERTIFIKATE_VERSCHLUESSELUNGEN"
    """Datenformate, Zertifikate, Verschlüsselungen"""
    DEBITORENMANAGEMENT = "DEBITORENMANAGEMENT"
    """Debitorenmanagement"""
    DEMAND_SIDE_MANAGEMENT = "DEMAND_SIDE_MANAGEMENT"
    """Demand-Side-Management"""
    EDI_VEREINBARUNG = "EDI_VEREINBARUNG"
    """EDI-Vereinbarung"""
    EDIFACT = "EDIFACT"
    """EDIFACT"""
    ENERGIEDATENMANAGEMENT = "ENERGIEDATENMANAGEMENT"
    """Energiedatenmanagement"""
    FAHRPLANMANAGEMENT = "FAHRPLANMANAGEMENT"
    """Fahrplanmanagement"""
    ALOCAT = "ALOCAT"
    """Format:ALOCAT"""
    APERAK = "APERAK"
    """Format:APERAK"""
    CONTRL = "CONTRL"
    """Format:CONTRL"""
    INVOIC = "INVOIC"
    """Format:INVOIC"""
    MSCONS = "MSCONS"
    """Format:MSCONS"""
    ORDERS = "ORDERS"
    """Format:ORDERS"""
    ORDERSP = "ORDERSP"
    """Format:ORDERSP"""
    REMADV = "REMADV"
    """Format:REMADV"""
    UTILMD = "UTILMD"
    """Format:UTILMD"""
    GABI = "GABI"
    """GaBi Gas"""
    GELI = "GELI"
    """GeLi Gas"""
    GERAETERUECKGABE = "GERAETERUECKGABE"
    """Geräterückgabe"""
    GERAETEWECHSEL = "GERAETEWECHSEL"
    """Gerätewechsel"""
    GPKE = "GPKE"
    """GPKE"""
    INBETRIEBNAHME = "INBETRIEBNAHME"
    """Inbetriebnahme"""
    KAPAZITAETSMANAGEMENT = "KAPAZITAETSMANAGEMENT"
    """Kapazitätsmanagement"""
    KLAERFAELLE = "KLAERFAELLE"
    """Klärfälle"""
    LASTGAENGE_RLM = "LASTGAENGE_RLM"
    """Lastgänge RLM"""
    LIEFERANTENRAHMENVERTRAG = "LIEFERANTENRAHMENVERTRAG"
    """Lieferantenrahmenvertrag"""
    LIEFERANTENWECHSEL = "LIEFERANTENWECHSEL"
    """Lieferantenwechsel"""
    MABIS = "MABIS"
    """MaBiS"""
    MAHNWESEN = "MAHNWESEN"
    """Mahnwesen"""
    MARKTGEBIETSVERANTWORTLICHER = "MARKTGEBIETSVERANTWORTLICHER"
    """Marktgebietsverantwortlicher"""
    MARKTKOMMUNIKATION = "MARKTKOMMUNIKATION"
    """Marktkommunikation"""
    MEHR_MINDERMENGEN = "MEHR_MINDERMENGEN"
    """Mehr- Mindermengen"""
    MSB_MDL = "MSB_MDL"
    """MSB - MDL"""
    NETZABRECHNUNG = "NETZABRECHNUNG"
    """Netzabrechnung"""
    NETZENTGELTE = "NETZENTGELTE"
    """Netzentgelte"""
    NETZMANAGEMENT = "NETZMANAGEMENT"
    """Netzmanagement"""
    RECHT = "RECHT"
    """Recht"""
    REGULIERUNGSMANAGEMENT = "REGULIERUNGSMANAGEMENT"
    """Regulierungsmanagement"""
    REKLAMATIONEN = "REKLAMATIONEN"
    """Reklamationen"""
    SPERREN_ENTSPERREN_INKASSO = "SPERREN_ENTSPERREN_INKASSO"
    """Sperren/Entsperren/Inkasso"""
    STAMMDATEN = "STAMMDATEN"
    """Stammdaten"""
    STOERUNGSFAELLE = "STOERUNGSFAELLE"
    """Übermittlung_Störungsfälle"""
    TECHNISCHE_FRAGEN = "TECHNISCHE_FRAGEN"
    """Technische Fragen"""
    UMSTELLUNG_INVOIC = "UMSTELLUNG_INVOIC"
    """Umstellung INVOIC"""
    VERSCHLUESSELUNG_SIGNATUR = "VERSCHLUESSELUNG_SIGNATUR"
    """Verschlüsselung/Signatur"""
    VERTRAGSMANAGEMENT = "VERTRAGSMANAGEMENT"
    """Vertragsmanagement"""
    VERTRIEB = "VERTRIEB"
    """Vertrieb"""
    WIM = "WIM"
    """WiM"""
    ZAEHLERSTAENDE_SLP = "ZAEHLERSTAENDE_SLP"
    """Zählerstände SLP"""
    ZAHLUNGSVERKEHR = "ZAHLUNGSVERKEHR"
    """Zahlungsverkehr"""
    ZUORDNUNGSVEREINBARUNG = "ZUORDNUNGSVEREINBARUNG"
    """Zuordnungsvereinbarung"""
    EINSPEISUNG = "EINSPEISUNG"
    """Einspeisung"""
    BEWEGUNGSDATEN = "BEWEGUNGSDATEN"
    """Bewegungsdaten"""



import pytest

from bo4e.com.lastprofil import Lastprofil
from bo4e.com.tagesparameter import Tagesparameter
from bo4e.enum.profilart import Profilart
from bo4e.enum.profilverfahren import Profilverfahren
from tests.serialization_helper import assert_serialization_roundtrip


class TestLastprofil:
    @pytest.mark.parametrize(
        "lastprofil",
        [
            pytest.param(
                Lastprofil(
                    bezeichnung="foo",
                    profilschar="foo2",
                    verfahren=Profilverfahren.SYNTHETISCH,
                    ist_einspeisung=True,
                    tagesparameter=Tagesparameter(
                        klimazone="7624q",
                        temperaturmessstelle="1234x",
                        dienstanbieter="ZT1",
                        herausgeber="BDEW",
                    ),
                    profilart=Profilart.ART_LASTPROFIL,
                    herausgeber="BDEW",
                )
            ),
        ],
    )
    def test_serialization_roundtrip(self, lastprofil: Lastprofil) -> None:
        """
        Test de-/serialisation
        """
        assert_serialization_roundtrip(lastprofil)



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class NetznutzungRechnungsart(StrEnum):
    """
    Abbildung verschiedener in der INVOIC angegebenen Rechnungsarten.
    """

    HANDELSRECHNUNG = "HANDELSRECHNUNG"
    """HANDELSRECHNUNG"""
    SELBSTAUSGESTELLT = "SELBSTAUSGESTELLT"
    """SELBSTAUSGESTELLT"""



"""
Contains tagesparameter class
"""

from typing import Annotated, Literal, Optional

from pydantic import Field

from bo4e.com.com import COM

from ..enum.comtyp import ComTyp


class Tagesparameter(COM):
    """
    Speichert Informationen zu einer tagesparameter abhängigen Messstelle. z.B. den Namen einer Klimazone oder die ID
    der Wetterstation für die Temperaturmessstelle
    """

    typ: Annotated[Literal[ComTyp.TAGESPARAMETER], Field(alias="_typ")] = ComTyp.TAGESPARAMETER

    klimazone: Optional[str] = None  #: Qualifier der Klimazone, e.g. 7624q
    temperaturmessstelle: Optional[str] = None  #: Qualifier der Temperaturmessstelle, e.g. 1234x
    dienstanbieter: Optional[str] = None  #: Dienstanbieter (bei Temperaturmessstellen), e.g. ZT1
    herausgeber: Optional[str] = None  #: Herausgeber des Lastprofil-Codes, e.g. BDEW



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


# pylint: disable=line-too-long
class Dienstleistungstyp(StrEnum):
    """
    Auflistung möglicher abzurechnender Dienstleistungen.
    """

    DATENBEREITSTELLUNG_TAEGLICH = "DATENBEREITSTELLUNG_TAEGLICH"
    """Datenbereitstellung täglich"""
    DATENBEREITSTELLUNG_WOECHENTLICH = "DATENBEREITSTELLUNG_WOECHENTLICH"
    """Datenbereitstellung wöchentlich"""
    DATENBEREITSTELLUNG_MONATLICH = "DATENBEREITSTELLUNG_MONATLICH"
    """Datenbereitstellung monatlich"""
    DATENBEREITSTELLUNG_JAEHRLICH = "DATENBEREITSTELLUNG_JAEHRLICH"
    """Datenbereitstellung jährlich"""
    DATENBEREITSTELLUNG_HISTORISCHE_LG = "DATENBEREITSTELLUNG_HISTORISCHE_LG"
    # Datenbereitstellung historischer Lastgänge
    DATENBEREITSTELLUNG_STUENDLICH = "DATENBEREITSTELLUNG_STUENDLICH"
    """Datenbereitstellung stündlich"""
    DATENBEREITSTELLUNG_VIERTELJAEHRLICH = "DATENBEREITSTELLUNG_VIERTELJAEHRLICH"
    # Datenbereitstellung vierteljährlich
    DATENBEREITSTELLUNG_HALBJAEHRLICH = "DATENBEREITSTELLUNG_HALBJAEHRLICH"
    """Datenbereitstellung halbjährlich"""
    DATENBEREITSTELLUNG_MONATLICH_ZUSAETZLICH = "DATENBEREITSTELLUNG_MONATLICH_ZUSAETZLICH"
    # Datenbereitstellung monatlich zusätzlich
    DATENBEREITSTELLUNG_EINMALIG = "DATENBEREITSTELLUNG_EINMALIG"
    """Datenbereitstellung einmalig"""
    AUSLESUNG_2X_TAEGLICH_FERNAUSLESUNG = "AUSLESUNG_2X_TAEGLICH_FERNAUSLESUNG"
    # Auslesung 2x täglich mittels Fernauslesung
    AUSLESUNG_TAEGLICH_FERNAUSLESUNG = "AUSLESUNG_TAEGLICH_FERNAUSLESUNG"
    """Auslesung täglich mittels Fernauslesung"""
    AUSLESUNG_MANUELL_MSB = "AUSLESUNG_MANUELL_MSB"
    """Auslesung, manuell vom Messstellenbetreiber vorgenommen"""
    AUSLESUNG_MONATLICH_FERNAUSLESUNG = "AUSLESUNG_MONATLICH_FERNAUSLESUNG"
    # Auslesung monatlich bei mittels Fernauslesung
    AUSLESUNG_JAEHRLICH_FERNAUSLESUNG = "AUSLESUNG_JAEHRLICH_FERNAUSLESUNG"
    # Auslesung jährlich bei SLP mittels Fernauslesung
    AUSLESUNG_MDE = "AUSLESUNG_MDE"
    """Auslesung mit mobiler Daten Erfassung (MDE)"""
    ABLESUNG_MONATLICH = "ABLESUNG_MONATLICH"
    """Ablesung monatlich"""
    ABLESUNG_VIERTELJAEHRLICH = "ABLESUNG_VIERTELJAEHRLICH"
    """Ablesung vierteljährlich"""
    ABLESUNG_HALBJAEHRLICH = "ABLESUNG_HALBJAEHRLICH"
    """Ablesung halbjährlich"""
    ABLESUNG_JAEHRLICH = "ABLESUNG_JAEHRLICH"
    """Ablesung jährlich"""
    AUSLESUNG_FERNAUSLESUNG = "AUSLESUNG_FERNAUSLESUNG"
    """Auslesung mittels Fernauslesung"""
    ABLESUNG_ZUSAETZLICH_MSB = "ABLESUNG_ZUSAETZLICH_MSB"
    """Ablesung, zusätzlich vom Messstellenbetreiber vorgenommen"""
    ABLESUNG_ZUSAETZLICH_KUNDE = "ABLESUNG_ZUSAETZLICH_KUNDE"
    """Ablesung SLP, zusätzlich vom Kunden vorgenommen"""
    AUSLESUNG_FERNAUSLESUNG_ZUSAETZLICH_MSB = "AUSLESUNG_FERNAUSLESUNG_ZUSAETZLICH_MSB"
    # Auslesung, mittels Fernauslesung, zusätzlich vom Messstellenbetreiber vorgenommen
    AUSLESUNG_MOATLICH_FERNAUSLESUNG = "AUSLESUNG_MOATLICH_FERNAUSLESUNG"
    """Auslesung monatlich mittels Fernauslesung"""
    AUSLESUNG_STUENDLICH_FERNAUSLESUNG = "AUSLESUNG_STUENDLICH_FERNAUSLESUNG"
    # Auslesung stündlich mittels Fernauslesung
    AUSLESUNG_TEMPERATURMENGENUMWERTER = "AUSLESUNG_TEMPERATURMENGENUMWERTER"
    """Auslesung Temperaturmengenumwerter"""
    AUSLESUNG_ZUSTANDSMENGENUMWERTER = "AUSLESUNG_ZUSTANDSMENGENUMWERTER"
    """Auslesung Zustandsmengenumwerter"""
    AUSLESUNG_SYSTEMMENGENUMWERTER = "AUSLESUNG_SYSTEMMENGENUMWERTER"
    """Auslesung Systemmengenumwerter"""
    AUSLESUNG_VORGANG = "AUSLESUNG_VORGANG"
    """Auslesung je Vorgang"""
    AUSLESUNG_KOMPAKTMENGENUMWERTER = "AUSLESUNG_KOMPAKTMENGENUMWERTER"
    """Auslesung Kompaktmengenumwerter"""
    SPERRUNG = "SPERRUNG"
    """Sperrung einer Abnahmestelle"""
    ENTSPERRUNG = "ENTSPERRUNG"
    """Entsperrung einer Abnahmestelle"""
    MAHNKOSTEN = "MAHNKOSTEN"
    """Mahnkosten"""
    INKASSOKOSTEN = "INKASSOKOSTEN"
    """Inkassokosten"""



from decimal import Decimal

import pytest

from bo4e import (
    Fremdkostenposition,
    Messpreistyp,
    Preis,
    Tarifberechnungsparameter,
    Tarifkalkulationsmethode,
    Tarifpreis,
)
from tests.serialization_helper import assert_serialization_roundtrip


class TestFremdkostenposition:
    @pytest.mark.parametrize(
        "tarifberechnungsparameter",
        [
            pytest.param(
                Tarifberechnungsparameter(
                    berechnungsmethode=Tarifkalkulationsmethode.ZONEN,
                    ist_messpreis_in_grundpreis_enthalten=True,
                    ist_messpreis_zu_beruecksichtigen=True,
                    messpreistyp=Messpreistyp.MESSPREIS_G6,
                    kw_inklusive=Decimal(12.5),
                    kw_weitere_mengen=Decimal(12.5),
                    hoechstpreis_n_t=Preis(),
                    hoechstpreis_h_t=Preis(),
                    mindestpreis=Preis(),
                    zusatzpreise=[Tarifpreis()],
                ),
                id="all attributes at first level",
            ),
        ],
    )
    def test_serialization_roundtrip(self, tarifberechnungsparameter: Fremdkostenposition) -> None:
        """
        Test de-/serialisation
        """
        assert_serialization_roundtrip(tarifberechnungsparameter)

    def test_missing_required_attribute(self) -> None:
        _ = Tarifberechnungsparameter()
        # ok, we're done. no exception here because there are no required attributes



from datetime import datetime, timezone
from decimal import Decimal

import pytest

from bo4e import Menge, Mengeneinheit, Vertragsteil
from tests.serialization_helper import assert_serialization_roundtrip


class TestVertragsteil:
    @pytest.mark.parametrize(
        "vertragsteil",
        [
            pytest.param(
                Vertragsteil(
                    vertragsteilbeginn=datetime(2001, 3, 15, tzinfo=timezone.utc),
                    vertragsteilende=datetime(2007, 11, 27, tzinfo=timezone.utc),
                    lokation="Bar",
                    vertraglich_fixierte_menge=Menge(wert=Decimal(3.1), einheit=Mengeneinheit.KWH),
                    minimale_abnahmemenge=Menge(wert=Decimal(2000), einheit=Mengeneinheit.KWH),
                    maximale_abnahmemenge=Menge(wert=Decimal(0.111111), einheit=Mengeneinheit.KWH),
                ),
            ),
        ],
    )
    def test_serialization_roundtrip(self, vertragsteil: Vertragsteil) -> None:
        """
        Test de-/serialisation of Vertragsteil.
        """
        assert_serialization_roundtrip(vertragsteil)



# pylint: disable=missing-module-docstring, line-too-long
from bo4e.enum.strenum import StrEnum


class Kalkulationsmethode(StrEnum):
    """
    Auflistung der verschiedenen Berechnungsmethoden für ein Preisblatt.
    """

    STUFEN = "STUFEN"
    """Stufenmodell, d.h. die Gesamtmenge wird in eine Stufe eingeordnet und für die gesamte Menge gilt der so ermittelte Preis"""
    ZONEN = "ZONEN"
    """Zonenmodell, d.h. die Gesamtmenge wird auf die Zonen aufgeteilt und für die Teilmengen gilt der jeweilige Preis der Zone."""
    VORZONEN_GP = "VORZONEN_GP"
    """Vorzonengrundpreis"""
    SIGMOID = "SIGMOID"
    """Sigmoidfunktion"""
    BLINDARBEIT_GT_50_PROZENT = "BLINDARBEIT_GT_50_PROZENT"
    """Blindarbeit oberhalb 50% der Wirkarbeit"""
    BLINDARBEIT_GT_40_PROZENT = "BLINDARBEIT_GT_40_PROZENT"
    """Blindarbeit oberhalb 40% der Wirkarbeit"""
    BLINDARBEIT_MIT_FREIMENGE = "BLINDARBEIT_MIT_FREIMENGE"
    """Blindarbeit. Freimenge ist durch cos phi oder prozentualem Anteil der Wirkarbeit angegeben."""
    AP_GP_ZONEN = "AP_GP_ZONEN"
    """Arbeits- und Grundpreis gezont"""
    LP_INSTALL_LEISTUNG = "LP_INSTALL_LEISTUNG"
    """Leistungsentgelt auf Grundlage der installierten Leistung"""
    AP_TRANSPORT_ODER_VERTEILNETZ = "AP_TRANSPORT_ODER_VERTEILNETZ"
    """AP auf Grundlage Transport- oder Verteilnetz"""
    AP_TRANSPORT_ODER_VERTEILNETZ_ORTSVERTEILNETZ_SIGMOID = "AP_TRANSPORT_ODER_VERTEILNETZ_ORTSVERTEILNETZ_SIGMOID"
    """AP auf Grundlage Transport- oder Verteilnetz, Ortsverteilnetz über Sigmoid"""
    LP_JAHRESVERBRAUCH = "LP_JAHRESVERBRAUCH"
    """Leistungsentgelt auf Grundlage des Jahresverbrauchs"""
    LP_TRANSPORT_ODER_VERTEILNETZ = "LP_TRANSPORT_ODER_VERTEILNETZ"
    """LP auf Grundlage Transport- oder Verteilnetz"""
    LP_TRANSPORT_ODER_VERTEILNETZ_ORTSVERTEILNETZ_SIGMOID = "LP_TRANSPORT_ODER_VERTEILNETZ_ORTSVERTEILNETZ_SIGMOID"
    """LP auf Grundlage Transport- oder Verteilnetz, Ortsverteilnetz über Sigmoid"""
    FUNKTIONEN = "FUNKTIONEN"
    """Funktionsbezogene Leistungsermittlung bei Verbräuchen oberhalb der SLP Grenze. (ähnlich Sigmoid)"""
    VERBRAUCH_UEBER_SLP_GRENZE_FUNKTIONSBEZOGEN_WEITERE_BERECHNUNG_ALS_LGK = (
        "VERBRAUCH_UEBER_SLP_GRENZE_FUNKTIONSBEZOGEN_WEITERE_BERECHNUNG_ALS_LGK"
    )
    """Bei einem Verbrauch über der SLP-Grenze (letzte Staffelgrenze überschritten) erfolgt die Berechnung funktionsbezogen (s.o.) als LGK."""



"""
Contains Konzessionsabgabe
"""

from decimal import Decimal
from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.abgabeart import AbgabeArt


# pylint: disable=too-few-public-methods, too-many-instance-attributes


@postprocess_docstring
class Konzessionsabgabe(COM):
    """
    Diese Komponente wird zur Übertagung der Details zu einer Konzessionsabgabe verwendet.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Konzessionsabgabe.svg" type="image/svg+xml"></object>

    .. HINT::
        `Konzessionsabgabe JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Konzessionsabgabe.json>`_

    """

    typ: Annotated[Literal[ComTyp.KONZESSIONSABGABE], Field(alias="_typ")] = ComTyp.KONZESSIONSABGABE

    satz: Optional["AbgabeArt"] = None
    """Art der Abgabe"""

    kosten: Optional[Decimal] = None
    """Konzessionsabgabe in E/kWh"""

    kategorie: Optional[str] = None
    """Gebührenkategorie der Konzessionsabgabe"""



from datetime import datetime
from decimal import Decimal

import pytest

from bo4e import Messwertstatus, Messwertstatuszusatz, Zeitraum, Zeitreihenwert
from tests.serialization_helper import assert_serialization_roundtrip


class TestZeitreihenwert:
    @pytest.mark.parametrize(
        "zeitreihenwert",
        [
            pytest.param(
                Zeitreihenwert(
                    zeitraum=Zeitraum(startdatum=datetime(2013, 5, 1), enddatum=datetime(2022, 1, 28)),
                    wert=Decimal(2.5),
                    status=Messwertstatus.ABGELESEN,
                    statuszusatz=Messwertstatuszusatz.Z78_GERAETEWECHSEL,
                ),
            ),
        ],
    )
    def test_serialization_roundtrip(self, zeitreihenwert: Zeitreihenwert) -> None:
        """
        Test de-/serialisation of Zeitreihenwert.
        """
        assert_serialization_roundtrip(zeitreihenwert)



"""
Contains Preis class
"""

from decimal import Decimal
from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.mengeneinheit import Mengeneinheit
    from ..enum.preisstatus import Preisstatus
    from ..enum.waehrungseinheit import Waehrungseinheit


# pylint: disable=too-few-public-methods


@postprocess_docstring
class Preis(COM):
    """
    Abbildung eines Preises mit Wert, Einheit, Bezugswert und Status.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Preis.svg" type="image/svg+xml"></object>

    .. HINT::
        `Preis JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Preis.json>`_

    """

    typ: Annotated[Literal[ComTyp.PREIS], Field(alias="_typ")] = ComTyp.PREIS

    wert: Optional[Decimal] = None
    """Gibt die nominale Höhe des Preises an."""
    einheit: Optional["Waehrungseinheit"] = None
    """Währungseinheit für den Preis, z.B. Euro oder Ct."""
    bezugswert: Optional["Mengeneinheit"] = None
    """Angabe, für welche Bezugsgröße der Preis gilt. Z.B. kWh."""

    status: Optional["Preisstatus"] = None
    """Gibt den Status des veröffentlichten Preises an"""



"""
Contains Tarifpreisposition class
"""

# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module
from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.mengeneinheit import Mengeneinheit
    from ..enum.preistyp import Preistyp
    from ..enum.waehrungseinheit import Waehrungseinheit
    from .preisstaffel import Preisstaffel


@postprocess_docstring
class Tarifpreisposition(COM):
    """
    Mit dieser Komponente können Tarifpreise verschiedener Typen abgebildet werden.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Tarifpreisposition.svg" type="image/svg+xml"></object>

    .. HINT::
        `Tarifpreisposition JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Tarifpreisposition.json>`_

    """

    typ: Annotated[Literal[ComTyp.TARIFPREISPOSITION], Field(alias="_typ")] = ComTyp.TARIFPREISPOSITION

    preistyp: Optional["Preistyp"] = None
    """Angabe des Preistypes (z.B. Grundpreis)"""
    einheit: Optional["Waehrungseinheit"] = None
    """Einheit des Preises (z.B. EURO)"""
    bezugseinheit: Optional["Mengeneinheit"] = None
    """Größe, auf die sich die Einheit bezieht, beispielsweise kWh, Jahr"""
    preisstaffeln: Optional[list["Preisstaffel"]] = None
    """Hier sind die Staffeln mit ihren Preisenangaben definiert"""

    mengeneinheitstaffel: Optional["Mengeneinheit"] = None
    """Gibt an, nach welcher Menge die vorgenannte Einschränkung erfolgt (z.B. Jahresstromverbrauch in kWh)"""



import pytest
from _decimal import Decimal

from bo4e import (
    Adresse,
    Bilanzierungsmethode,
    Energierichtung,
    Lastgang,
    Marktlokation,
    Menge,
    Mengeneinheit,
    Messlokation,
    Netzebene,
    Sparte,
    Zeitreihenwert,
)
from tests.serialization_helper import assert_serialization_roundtrip


class TestLastgang:
    @pytest.mark.parametrize(
        "lastgang",
        [
            pytest.param(
                Lastgang(
                    version="1.1",
                    sparte=Sparte.STROM,
                    obis_kennzahl="1-0:1.8.1",
                    messgroesse=Mengeneinheit.KWH,
                    werte=[Zeitreihenwert()],
                    marktlokation=Marktlokation(
                        marktlokations_id="51238696781",
                        sparte=Sparte.GAS,
                        lokationsadresse=Adresse(
                            postleitzahl="04177", ort="Leipzig", hausnummer="1", strasse="Jahnalle"
                        ),
                        energierichtung=Energierichtung.EINSP,
                        bilanzierungsmethode=Bilanzierungsmethode.PAUSCHAL,
                        netzebene=Netzebene.NSP,
                    ),
                    messlokation=Messlokation(
                        messlokations_id="DE00056266802AO6G56M11SN51G21M24S",
                        sparte=Sparte.STROM,
                    ),
                    zeit_intervall_laenge=Menge(wert=Decimal(1.0), einheit=Mengeneinheit.VIERTEL_STUNDE),
                ),
            ),
        ],
    )
    def test_serialization_roundtrip(self, lastgang: Lastgang) -> None:
        """
        Test de-/serialisation of Lastgang.
        """
        assert_serialization_roundtrip(lastgang)



from datetime import datetime, timezone
from decimal import Decimal

import pytest

from bo4e import Mengeneinheit, Vertragskonditionen, Zeitraum
from tests.serialization_helper import assert_serialization_roundtrip


class TestVertragskonditionen:
    @pytest.mark.parametrize(
        "vertragskonditionen",
        [
            pytest.param(
                Vertragskonditionen(
                    beschreibung="Foobar",
                    anzahl_abschlaege=Decimal(3),
                    vertragslaufzeit=Zeitraum(
                        startdatum=datetime(2012, 9, 21, tzinfo=timezone.utc),
                        enddatum=datetime(2013, 10, 11, tzinfo=timezone.utc),
                    ),
                    kuendigungsfrist=Zeitraum(dauer="P3W"),
                    vertragsverlaengerung=Zeitraum(dauer="P14D"),
                    abschlagszyklus=Zeitraum(dauer="P5D"),
                ),
            ),
            pytest.param(
                Vertragskonditionen(),
            ),
        ],
    )
    def test_serialization_roundtrip(self, vertragskonditionen: Vertragskonditionen) -> None:
        """
        Test de-/serialisation of Vertragskonditionen.
        """
        assert_serialization_roundtrip(vertragskonditionen)



"""
Contains Angebotsteil class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..bo.marktlokation import Marktlokation
    from .angebotsposition import Angebotsposition
    from .betrag import Betrag
    from .menge import Menge
    from .zeitraum import Zeitraum

# pylint: disable=too-few-public-methods
# pylint: disable=no-name-in-module


@postprocess_docstring
class Angebotsteil(COM):
    """
    Mit dieser Komponente wird ein Teil einer Angebotsvariante abgebildet.
    Hier werden alle Angebotspositionen aggregiert.
    Angebotsteile werden im einfachsten Fall für eine Marktlokation oder Lieferstellenadresse erzeugt.
    Hier werden die Mengen und Gesamtkosten aller Angebotspositionen zusammengefasst.
    Eine Variante besteht mindestens aus einem Angebotsteil.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Angebotsteil.svg" type="image/svg+xml"></object>

    .. HINT::
        `Angebotsteil JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Angebotsteil.json>`_

    """

    typ: Annotated[Literal[ComTyp.ANGEBOTSTEIL], Field(alias="_typ")] = ComTyp.ANGEBOTSTEIL

    positionen: Optional[list["Angebotsposition"]] = None
    """Einzelne Positionen, die zu diesem Angebotsteil gehören"""

    anfrage_subreferenz: Optional[str] = None
    """Identifizierung eines Subkapitels einer Anfrage, beispielsweise das Los einer Ausschreibung"""
    lieferstellenangebotsteil: Optional[list["Marktlokation"]] = None
    """
    Marktlokationen, für die dieses Angebotsteil gilt, falls vorhanden.
    Durch die Marktlokation ist auch die Lieferadresse festgelegt
    """
    gesamtmengeangebotsteil: Optional["Menge"] = None
    """Summe der Verbräuche aller in diesem Angebotsteil eingeschlossenen Lieferstellen"""
    gesamtkostenangebotsteil: Optional["Betrag"] = None
    """Summe der Jahresenergiekosten aller in diesem Angebotsteil enthaltenen Lieferstellen"""
    lieferzeitraum: Optional["Zeitraum"] = None
    """Hier kann der Belieferungszeitraum angegeben werden, für den dieser Angebotsteil gilt"""



"""
Contains RegionalerAufAbschlag class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp

# pylint: disable=R0801
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.aufabschlagstyp import AufAbschlagstyp
    from ..enum.aufabschlagsziel import AufAbschlagsziel
    from ..enum.waehrungseinheit import Waehrungseinheit
    from .energiemix import Energiemix
    from .preisgarantie import Preisgarantie
    from .regionalepreisstaffel import RegionalePreisstaffel
    from .tarifeinschraenkung import Tarifeinschraenkung
    from .vertragskonditionen import Vertragskonditionen
    from .zeitraum import Zeitraum

# pylint: disable=too-few-public-methods, too-many-instance-attributes
# pylint: disable=no-name-in-module


@postprocess_docstring
class RegionalerAufAbschlag(COM):
    """
    Mit dieser Komponente können Auf- und Abschläge verschiedener Typen im Zusammenhang mit regionalen Gültigkeiten
    abgebildet werden.
    Hier sind auch die Auswirkungen auf verschiedene Tarifparameter modelliert, die sich durch die Auswahl eines Auf-
    oder Abschlags ergeben.

    .. raw:: html

        <object data="../_static/images/bo4e/com/RegionalerAufAbschlag.svg" type="image/svg+xml"></object>

    .. HINT::
        `RegionalerAufAbschlag JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/RegionalerAufAbschlag.json>`_

    """

    typ: Annotated[Literal[ComTyp.REGIONALERAUFABSCHLAG], Field(alias="_typ")] = ComTyp.REGIONALERAUFABSCHLAG

    bezeichnung: Optional[str] = None
    """Bezeichnung des Auf-/Abschlags"""

    staffeln: Optional[list["RegionalePreisstaffel"]] = None
    """Werte für die gestaffelten Auf/Abschläge mit regionaler Eingrenzung"""

    beschreibung: Optional[str] = None
    """Beschreibung des Auf-/Abschlags"""

    auf_abschlagstyp: Optional["AufAbschlagstyp"] = None
    """Typ des Aufabschlages (z.B. absolut oder prozentual)"""

    auf_abschlagsziel: Optional["AufAbschlagsziel"] = None
    """Diesem Preis oder den Kosten ist der Auf/Abschlag zugeordnet. Z.B. Arbeitspreis, Gesamtpreis etc."""

    einheit: Optional["Waehrungseinheit"] = None
    """Gibt an in welcher Währungseinheit der Auf/Abschlag berechnet wird (nur im Falle absoluter Aufschlagstypen)."""

    website: Optional[str] = None
    """Internetseite, auf der die Informationen zum Auf-/Abschlag veröffentlicht sind"""

    zusatzprodukte: Optional[list[str]] = None
    """Zusatzprodukte, die nur in Kombination mit diesem AufAbschlag erhältlich sind"""

    voraussetzungen: Optional[list[str]] = None
    """Voraussetzungen, die erfüllt sein müssen, damit dieser AufAbschlag zur Anwendung kommen kann"""

    tarifnamensaenderungen: Optional[str] = None
    """Durch die Anwendung des Auf/Abschlags kann eine Änderung des Tarifnamens auftreten"""

    gueltigkeitszeitraum: Optional["Zeitraum"] = None
    """Zeitraum, in dem der Abschlag zur Anwendung kommen kann"""

    energiemixaenderung: Optional["Energiemix"] = None
    """
    Der Energiemix kann sich durch einen AufAbschlag ändern (z.B. zwei Cent Aufschlag für Ökostrom).
    Sollte dies der Fall sein, wird hier die neue Zusammensetzung des Energiemix angegeben.
    """

    vertagskonditionsaenderung: Optional["Vertragskonditionen"] = None
    """
    Änderungen in den Vertragskonditionen;
    Falls in dieser Komponenten angegeben, werden die Tarifparameter hiermit überschrieben.
    """

    garantieaenderung: Optional["Preisgarantie"] = None
    """
    Änderungen in den Garantievereinbarungen;
    Falls in dieser Komponenten angegeben, werden die Tarifparameter hiermit überschrieben.
    """

    einschraenkungsaenderung: Optional["Tarifeinschraenkung"] = None
    """
    Änderungen in den Einschränkungen zum Tarif;
    Falls in dieser Komponenten angegeben, werden die Tarifparameter hiermit überschrieben.
    """



"""
Contains KriteriumWert class
"""

# pylint: disable=too-few-public-methods
from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.tarifregionskriterium import Tarifregionskriterium


@postprocess_docstring
class KriteriumWert(COM):
    """
    Mit dieser Komponente können Kriterien und deren Werte definiert werden

    .. raw:: html

        <object data="../_static/images/bo4e/com/KriteriumWert.svg" type="image/svg+xml"></object>

    .. HINT::
        `KriteriumWert JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/KriteriumWert.json>`_

    """

    typ: Annotated[Literal[ComTyp.KRITERIUMWERT], Field(alias="_typ")] = ComTyp.KRITERIUMWERT

    kriterium: Optional["Tarifregionskriterium"] = None
    """Hier steht, für welches Kriterium der Wert gilt. Z.B. Postleitzahlen"""
    wert: Optional[str] = None
    """Ein Wert, passend zum Kriterium. Z.B. eine Postleitzahl."""



from decimal import Decimal

import pytest

from bo4e import (
    EMobilitaetsart,
    Erzeugungsart,
    Lokationszuordnung,
    Menge,
    Speicherart,
    TechnischeRessource,
    TechnischeRessourceNutzung,
    TechnischeRessourceVerbrauchsart,
    Waermenutzung,
)
from tests.serialization_helper import assert_serialization_roundtrip


class TestTechnischeRessource:
    @pytest.mark.parametrize(
        "technische_ressource",
        [
            pytest.param(
                TechnischeRessource(
                    technische_ressource_id="3784658734657",
                    vorgelagerte_messlokation_id="37846587343434",
                    zugeordnete_marktlokation_id="12242432423",
                    zugeordnete_steuerbare_ressource_id="281238912739",
                    nennleistungaufnahme=Menge(wert=Decimal(1000.0)),
                    nennleistungabgabe=Menge(wert=Decimal(1000.0)),
                    speicherkapazitaet=Menge(wert=Decimal(1000.0)),
                    technische_ressource_nutzung=TechnischeRessourceNutzung.SPEICHER,
                    technische_ressource_verbrauchsart=TechnischeRessourceVerbrauchsart.KRAFT_LICHT,
                    waermenutzung=Waermenutzung.SPEICHERHEIZUNG,
                    emobilitaetsart=EMobilitaetsart.WALLBOX,
                    erzeugungsart=Erzeugungsart.WIND,
                    speicherart=Speicherart.WASSERSTOFFSPEICHER,
                    lokationsbuendel_objektcode="9992 00000 125 6",
                    lokationszuordnungen=[Lokationszuordnung()],
                ),
            ),
        ],
    )
    def test_serialization_roundtrip(self, technische_ressource: TechnischeRessource) -> None:
        """
        Test de-/serialisation of TechnischeRessource.
        """
        assert_serialization_roundtrip(technische_ressource)



import pytest

from bo4e import Standorteigenschaften, StandorteigenschaftenGas, StandorteigenschaftenStrom
from tests.serialization_helper import assert_serialization_roundtrip


class TestStandorteigenschaften:
    @pytest.mark.parametrize(
        "standorteigenschaften",
        [
            pytest.param(
                Standorteigenschaften(
                    eigenschaften_strom=[StandorteigenschaftenStrom()],
                    eigenschaften_gas=StandorteigenschaftenGas(),
                )
            ),
        ],
    )
    def test_serialization_roundtrip(self, standorteigenschaften: Standorteigenschaften) -> None:
        """
        Test de-/serialisation of Sigmoidparameter with minimal attributes.
        """
        assert_serialization_roundtrip(standorteigenschaften)



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Vertragsstatus(StrEnum):
    """
    Abbildung einer Statusinformation für Verträge.
    """

    IN_ARBEIT = "IN_ARBEIT"
    """in Arbeit"""
    UEBERMITTELT = "UEBERMITTELT"
    """übermittelt"""
    ANGENOMMEN = "ANGENOMMEN"
    """angenommen"""
    AKTIV = "AKTIV"
    """aktiv"""
    ABGELEHNT = "ABGELEHNT"
    """abgelehnt"""
    WIDERRUFEN = "WIDERRUFEN"
    """widerrufen"""
    STORNIERT = "STORNIERT"
    """storniert"""
    GEKUENDIGT = "GEKUENDIGT"
    """gekündigt"""
    BEENDET = "BEENDET"
    """beendet"""



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class TechnischeRessourceVerbrauchsart(StrEnum):
    """
    Verbrauchsart der technischen Ressource
    """

    KRAFT_LICHT = "KRAFT_LICHT"
    WAERME = "WAERME"
    E_MOBILITAET = "E_MOBILITAET"
    STRASSENBELEUCHTUNG = "STRASSENBELEUCHTUNG"



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class SteuerkanalLeistungsbeschreibung(StrEnum):
    """
    Beschreibung des Steuerkanals
    """

    AN_AUS = "AN_AUS"
    GESTUFT = "GESTUFT"



"""
Contains AufAbschlag class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.aufabschlagstyp import AufAbschlagstyp
    from ..enum.aufabschlagsziel import AufAbschlagsziel
    from ..enum.waehrungseinheit import Waehrungseinheit
    from .preisstaffel import Preisstaffel
    from .zeitraum import Zeitraum

# pylint: disable=too-few-public-methods, too-many-instance-attributes


@postprocess_docstring
class AufAbschlag(COM):
    """
    Modell für die preiserhöhenden (Aufschlag) bzw. preisvermindernden (Abschlag) Zusatzvereinbarungen,
    die individuell zu einem neuen oder bestehenden Liefervertrag abgeschlossen wurden.

    .. raw:: html

        <object data="../_static/images/bo4e/com/AufAbschlag.svg" type="image/svg+xml"></object>

    .. HINT::
        `AufAbschlag JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/AufAbschlag.json>`_

    """

    typ: Annotated[Literal[ComTyp.AUFABSCHLAG], Field(alias="_typ")] = ComTyp.AUFABSCHLAG

    bezeichnung: Optional[str] = None
    """Bezeichnung des Auf-/Abschlags"""
    staffeln: Optional[list["Preisstaffel"]] = None
    """Werte für die gestaffelten Auf/Abschläge."""

    beschreibung: Optional[str] = None
    """Beschreibung zum Auf-/Abschlag"""
    auf_abschlagstyp: Optional["AufAbschlagstyp"] = None
    """Typ des Aufabschlages (z.B. absolut oder prozentual)."""
    auf_abschlagsziel: Optional["AufAbschlagsziel"] = None
    """Diesem Preis oder den Kosten ist der Auf/Abschlag zugeordnet. Z.B. Arbeitspreis, Gesamtpreis etc.."""
    einheit: Optional["Waehrungseinheit"] = None
    """ Gibt an in welcher Währungseinheit der Auf/Abschlag berechnet wird. Euro oder Ct..
    (Nur im Falle absoluter Aufschlagstypen). """
    website: Optional[str] = None
    """Internetseite, auf der die Informationen zum Auf-/Abschlag veröffentlicht sind."""
    gueltigkeitszeitraum: Optional["Zeitraum"] = None
    """Internetseite, auf der die Informationen zum Auf-/Abschlag veröffentlicht sind."""



import pytest

from bo4e import KriteriumWert, Tarifregionskriterium
from tests.serialization_helper import assert_serialization_roundtrip


class TestKriteriumWert:
    @pytest.mark.parametrize(
        "kriteriumwert",
        [
            pytest.param(
                KriteriumWert(
                    kriterium=Tarifregionskriterium.ORT,
                    wert="Grünwald",
                ),
            ),
        ],
    )
    def test_kriteriumwert_serialization_roundtrip(self, kriteriumwert: KriteriumWert) -> None:
        """
        Test de-/serialisation of KriteriumWert.
        """
        assert_serialization_roundtrip(kriteriumwert)



"""
Contains Lokationszuordnung class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

if TYPE_CHECKING:
    from ..bo.marktlokation import Marktlokation
    from ..bo.messlokation import Messlokation
    from ..bo.netzlokation import Netzlokation
    from ..bo.steuerbareressource import SteuerbareRessource
    from ..bo.technischeressource import TechnischeRessource
    from ..com.zeitraum import Zeitraum


@postprocess_docstring
class Lokationszuordnung(Geschaeftsobjekt):
    """
    Modell für die Abbildung der Referenz auf die Lokationsbündelstruktur. Diese gibt an welche Marktlokationen,
    Messlokationen, Netzlokationen, technische/steuerbaren Ressourcen an einer Lokation vorhanden sind.

    .. raw:: html

        <object data="../_static/images/bo4e/bo/Lokationszuordnung.svg" type="image/svg+xml"></object>

    .. HINT::
        `Lokationszuordnung JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/Lokationszuordnung.json>`_
    """

    typ: Annotated[Literal[BoTyp.LOKATIONSZUORDNUNG], Field(alias="_typ")] = BoTyp.LOKATIONSZUORDNUNG

    marktlokationen: Optional[list["Marktlokation"]] = None
    """Liste mit referenzierten Marktlokationen"""
    messlokationen: Optional[list["Messlokation"]] = None
    """Liste mit referenzierten Messlokationen"""
    netzlokationen: Optional[list["Netzlokation"]] = None
    """Liste mit referenzierten Netzlokationen"""
    technische_ressourcen: Optional[list["TechnischeRessource"]] = None
    """Liste mit referenzierten technischen Ressourcen"""
    steuerbare_ressourcen: Optional[list["SteuerbareRessource"]] = None
    """Liste mit referenzierten steuerbaren Ressourcen"""
    gueltigkeit: Optional["Zeitraum"] = None
    """Zeitraum der Gültigkeit"""
    zuordnungstyp: Optional[str] = None
    """Verknüpfungsrichtung z.B. Malo-Melo [TODO: Eventuell anderer Datentyp]"""
    lokationsbuendelcode: Optional[str] = None
    """Code, der angibt wie die Lokationsbündelstruktur zusammengesetzt ist (zu finden unter "Codeliste der Lokationsbündelstrukturen" auf https://www.edi-energy.de/index.php?id=38)"""



"""
strenum contains an enum that inherits from the plain enum and string.
"""

from enum import Enum


# pylint: disable=too-few-public-methods
class StrEnum(str, Enum):
    """
    An enum that has string values.
    """

    # see https://docs.python.org/3/library/enum.html?highlight=strenum#others



import pytest

from bo4e import Marktrolle, SteuerbareRessource
from bo4e.bo.lokationszuordnung import Lokationszuordnung
from bo4e.com.konfigurationsprodukt import Konfigurationsprodukt
from bo4e.enum.steuerkanalleistungsbeschreibung import SteuerkanalLeistungsbeschreibung
from tests.serialization_helper import assert_serialization_roundtrip


class TestSteuerbareRessource:
    @pytest.mark.parametrize(
        "steuerbare_ressource",
        [
            pytest.param(
                SteuerbareRessource(
                    steuerbare_ressource_id="3784658734657",
                    steuerkanal_leistungsbeschreibung=SteuerkanalLeistungsbeschreibung.AN_AUS,
                    zugeordnete_msb_codenummer="1829371872392",
                    konfigurationsprodukte=[Konfigurationsprodukt()],
                    eigenschaft_msb_lokation=Marktrolle.LF,
                    lokationsbuendel_objektcode="9992 00000 125 6",
                    lokationszuordnungen=[Lokationszuordnung()],
                ),
            ),
        ],
    )
    def test_serialization_roundtrip(self, steuerbare_ressource: SteuerbareRessource) -> None:
        """
        Test de-/serialisation of SteuerbareRessource.
        """
        assert_serialization_roundtrip(steuerbare_ressource)



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class NetznutzungRechnungstyp(StrEnum):
    """
    Abbildung verschiedener in der INVOIC angegebenen Rechnungstypen.
    """

    ABSCHLUSSRECHNUNG = "ABSCHLUSSRECHNUNG"
    """ABSCHLUSSRECHNUNG"""
    ABSCHLAGSRECHNUNG = "ABSCHLAGSRECHNUNG"
    """ABSCHLAGSRECHNUNG"""
    TURNUSRECHNUNG = "TURNUSRECHNUNG"
    """TURNUSRECHNUNG"""
    MONATSRECHNUNG = "MONATSRECHNUNG"
    """MONATSRECHNUNG"""
    WIMRECHNUNG = "WIMRECHNUNG"
    """WIMRECHNUNG"""
    ZWISCHENRECHNUNG = "ZWISCHENRECHNUNG"
    """ZWISCHENRECHNUNG"""
    INTEGRIERTE_13TE_RECHNUNG = "INTEGRIERTE_13TE_RECHNUNG"
    """INTEGRIERTE_13TE_RECHNUNG"""
    ZUSAETZLICHE_13TE_RECHNUNG = "ZUSAETZLICHE_13TE_RECHNUNG"
    """ZUSAETZLICHE_13TE_RECHNUNG"""
    MEHRMINDERMENGENRECHNUNG = "MEHRMINDERMENGENRECHNUNG"
    """MEHRMINDERMENGENRECHNUNG"""



"""
Contains technische Ressource class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .geschaeftsobjekt import Geschaeftsobjekt

if TYPE_CHECKING:
    from ..bo.lokationszuordnung import Lokationszuordnung
    from ..com.menge import Menge
    from ..enum.emobilitaetsart import EMobilitaetsart
    from ..enum.erzeugungsart import Erzeugungsart
    from ..enum.speicherart import Speicherart
    from ..enum.technischeressourcenutzung import TechnischeRessourceNutzung
    from ..enum.technischeressourceverbrauchsart import TechnischeRessourceVerbrauchsart
    from ..enum.waermenutzung import Waermenutzung

# pylint: disable=too-many-instance-attributes, too-few-public-methods


@postprocess_docstring
class TechnischeRessource(Geschaeftsobjekt):
    """
    Object containing information about a technische Ressource

    .. raw:: html

        <object data="../_static/images/bo4e/bo/TechnischeRessource.svg" type="image/svg+xml"></object>

    .. HINT::
        `TechnischeRessource JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/TechnischeRessource.json>`_

    """

    typ: Annotated[Literal[BoTyp.TECHNISCHERESSOURCE], Field(alias="_typ")] = BoTyp.TECHNISCHERESSOURCE

    technische_ressource_id: Optional[str] = None
    """Identifikationsnummer einer technischen Ressource"""
    vorgelagerte_messlokation_id: Optional[str] = None
    """Vorgelagerte Messlokation ID"""
    zugeordnete_marktlokation_id: Optional[str] = None
    """Referenz auf die der technischen Ressource zugeordneten Marktlokation"""
    zugeordnete_steuerbare_ressource_id: Optional[str] = None
    """Referenz auf die der technischen Ressource zugeordneten Steuerbaren Ressource"""
    nennleistungaufnahme: Optional["Menge"] = None
    """Nennleistung (Aufnahme)"""
    nennleistungabgabe: Optional["Menge"] = None
    """Nennleistung (Abgabe)"""
    speicherkapazitaet: Optional["Menge"] = None
    """Speicherkapazität"""
    technische_ressource_nutzung: Optional["TechnischeRessourceNutzung"] = None
    """Art und Nutzung der technischen Ressource"""
    technische_ressource_verbrauchsart: Optional["TechnischeRessourceVerbrauchsart"] = None
    """Verbrauchsart der technischen Ressource"""
    waermenutzung: Optional["Waermenutzung"] = None
    """Wärmenutzung"""
    emobilitaetsart: Optional["EMobilitaetsart"] = None
    """Art der E-Mobilität"""
    erzeugungsart: Optional["Erzeugungsart"] = None
    """Art der Erzeugung der Energie"""
    speicherart: Optional["Speicherart"] = None
    """Art des Speichers"""
    lokationszuordnungen: Optional[list["Lokationszuordnung"]] = None
    """Lokationszuordnung, um bspw. die zugehörigen Messlokationen anzugeben"""
    lokationsbuendel_objektcode: Optional[str] = None
    """Lokationsbuendel Code, der die Funktion dieses BOs an der Lokationsbuendelstruktur beschreibt."""



"""
Contains Enums for Zeitreihentyp.
"""

from bo4e.enum.strenum import StrEnum


class Zeitreihentyp(StrEnum):
    """
    Codes der Summenzeitreihentypen.

    Die nachfolgenden Codes sind in DE7111 zu nutzen:
    https://www.edi-energy.de/index.php?id=38&tx_bdew_bdew%5Buid%5D=695&tx_bdew_bdew%5Baction%5D=download
    &tx_bdew_bdew%5Bcontroller%5D=Dokument&cHash=67782e05d8b0f75fbe3a0e1801d07ed0
    """

    EGS = "EGS"  #: Einspeisegangsumme
    LGS = "LGS"  #: Lastgangsumme
    NZR = "NZR"  #: Netzzeitreihe
    SES = "SES"  #: Standardeinspeiseprofilsumme
    SLS = "SLS"  #: Standardlastsumme
    TES = "TES"  #: tagesparameterabhängige Einspeiseprofilsumme
    TLS = "TLS"  #: tagesparameterabhängige Lastprofilsumme
    SLS_TLS = "SLS_TLS"  #: gemeinsame Messung aus SLS und TLS
    SES_TES = "SES_TES"  #: gemeinsame Messung aus SES und TES



"""
Contains Enums for Profiltyp.
"""

from bo4e.enum.strenum import StrEnum


class Profiltyp(StrEnum):
    """
    Profiltyp (temperaturabhängig / Standardlastprofil).
    """

    SLP_SEP = "SLP_SEP"  #: SLP/SEP
    TLP_TEP = "TLP_TEP"  #: TLP/TEP



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Mengeneinheit(StrEnum):
    """
    Einheit: Messgrößen, die per Messung oder Vorgabe ermittelt werden können.
    """

    W = "W"
    """Watt"""
    WH = "WH"
    """Wattstunde"""
    KW = "KW"
    """Kilowatt"""
    KWH = "KWH"
    """Kilowattstunde"""
    KVARH = "KVARH"
    """Kilovarstunde"""
    MW = "MW"
    """Megawatt"""
    MWH = "MWH"
    """Megawattstunde"""
    STUECK = "STUECK"
    """Stückzahl"""
    KUBIKMETER = "KUBIKMETER"
    """Kubikmeter (Gas)"""
    SEKUNDE = "SEKUNDE"
    """Sekunde"""
    MINUTE = "MINUTE"
    """Minute"""
    STUNDE = "STUNDE"
    """Stunde"""
    VIERTEL_STUNDE = "VIERTEL_STUNDE"
    """Viertelstunde"""
    TAG = "TAG"
    """Tag"""
    WOCHE = "WOCHE"
    """Woche"""
    MONAT = "MONAT"
    """Monat"""
    QUARTAL = "QUARTAL"
    """Quartal"""
    HALBJAHR = "HALBJAHR"
    """Halbjahr"""
    JAHR = "JAHR"
    """Jahr"""
    PROZENT = "PROZENT"
    """Prozent"""
    KVAR = "KVAR"
    """Kilovar"""
    KWHK = "KWHK"
    """Kilowattstunde pro Kelvin"""
    VAR = "VAR"
    """Var"""
    VARH = "VARH"
    """Varstunde"""



"""
Contains PositionsAufAbschlag
"""

from decimal import Decimal

# pylint: disable=too-few-public-methods
from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.aufabschlagstyp import AufAbschlagstyp
    from ..enum.waehrungseinheit import Waehrungseinheit


@postprocess_docstring
class PositionsAufAbschlag(COM):
    """
    Differenzierung der zu betrachtenden Produkte anhand der preiserhöhenden (Aufschlag)
    bzw. preisvermindernden (Abschlag) Zusatzvereinbarungen,
    die individuell zu einem neuen oder bestehenden Liefervertrag abgeschlossen werden können.
    Es können mehrere Auf-/Abschläge gleichzeitig ausgewählt werden.

    .. raw:: html

        <object data="../_static/images/bo4e/com/PositionsAufAbschlag.svg" type="image/svg+xml"></object>

    .. HINT::
        `PositionsAufAbschlag JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/PositionsAufAbschlag.json>`_

    """

    typ: Annotated[Literal[ComTyp.POSITIONSAUFABSCHLAG], Field(alias="_typ")] = ComTyp.POSITIONSAUFABSCHLAG

    bezeichnung: Optional[str] = None
    """Bezeichnung des Auf-/Abschlags"""
    beschreibung: Optional[str] = None
    """Beschreibung zum Auf-/Abschlag"""
    auf_abschlagstyp: Optional["AufAbschlagstyp"] = None
    """Typ des AufAbschlages"""
    auf_abschlagswert: Optional[Decimal] = None
    """Höhe des Auf-/Abschlages"""
    auf_abschlagswaehrung: Optional["Waehrungseinheit"] = None
    """Einheit, in der der Auf-/Abschlag angegeben ist (z.B. ct/kWh)."""



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Messgroesse(StrEnum):
    """
    Gibt die physikalische Größe an, die gemessen wurde.
    """

    STROM = "STROM"
    """Strom"""
    SPANNUNG = "SPANNUNG"
    """Spannung"""
    WIRKLEISTUNG = "WIRKLEISTUNG"
    """Wirkleistung"""
    BLINDLEISTUNG = "BLINDLEISTUNG"
    """Blindleistung"""
    DRUCK = "DRUCK"
    """Druck"""
    LASTGANG = "LASTGANG"
    """Lastgang"""
    LASTPROFIL = "LASTPROFIL"
    """Lastprofil"""
    TEMPERATUR = "TEMPERATUR"
    """Temperatur"""
    ZZAHL = "ZZAHL"
    """Zustandszahl"""
    BRENNWERT = "BRENNWERT"
    """Brennwert"""
    GRADTZAGSZAHLEN = "GRADTZAGSZAHLEN"
    """Gradtagszahlen"""
    VOLUMENSTROM = "VOLUMENSTROM"
    """Volumenstrom"""
    PREISE = "PREISE"
    """Preise"""



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Vertragsform(StrEnum):
    """
    Aufzählung der Möglichkeiten zu Vertragsformen in Ausschreibungen.
    """

    ONLINE = "ONLINE"
    """Online"""
    DIREKT = "DIREKT"
    """Direkt"""
    FAX = "FAX"
    """Auftragsfax"""



"""
Contains Tarifberechnungsparameter class
"""

from decimal import Decimal
from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.comtyp import ComTyp
from ..utils import postprocess_docstring
from .com import COM

if TYPE_CHECKING:
    from ..enum.messpreistyp import Messpreistyp
    from ..enum.tarifkalkulationsmethode import Tarifkalkulationsmethode
    from .preis import Preis
    from .tarifpreis import Tarifpreis

# yes. there is no description in the official docs.
# https://github.com/Hochfrequenz/BO4E-python/issues/328

# pylint: disable=too-few-public-methods, empty-docstring, too-many-instance-attributes


@postprocess_docstring
class Tarifberechnungsparameter(COM):
    """
    In dieser Komponente sind die Berechnungsparameter für die Ermittlung der Tarifkosten zusammengefasst.

    .. raw:: html

        <object data="../_static/images/bo4e/com/Tarifberechnungsparameter.svg" type="image/svg+xml"></object>

    .. HINT::
        `Tarifberechnungsparameter JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/com/Tarifberechnungsparameter.json>`_

    """

    typ: Annotated[Literal[ComTyp.TARIFBERECHNUNGSPARAMETER], Field(alias="_typ")] = ComTyp.TARIFBERECHNUNGSPARAMETER

    berechnungsmethode: Optional["Tarifkalkulationsmethode"] = None
    """Gibt an, wie die Einzelpreise des Tarifes zu verarbeiten sind"""
    ist_messpreis_in_grundpreis_enthalten: Optional[bool] = None
    """True, falls der Messpreis im Grundpreis (GP) enthalten ist"""

    ist_messpreis_zu_beruecksichtigen: Optional[bool] = None
    """
    True, falls bei der Bildung des Durchschnittspreises für die Höchst- und Mindestpreisbetrachtung der Messpreis mit
    berücksichtigt wird
    """

    messpreistyp: Optional["Messpreistyp"] = None
    """Typ des Messpreises"""

    kw_inklusive: Optional[Decimal] = None
    """Im Preis bereits eingeschlossene Leistung (für Gas)"""
    # todo: type decimal is most likely wrong: https://github.com/Hochfrequenz/BO4E-python/issues/327

    kw_weitere_mengen: Optional[Decimal] = None
    """Intervall, indem die über "kwInklusive" hinaus abgenommene Leistung kostenpflichtig wird (z.B. je 5 kW 20 EURO)"""
    # todo: type decimal is most likely wrong: https://github.com/Hochfrequenz/BO4E-python/issues/327

    hoechstpreis_n_t: Optional["Preis"] = None
    """Höchstpreis für den Durchschnitts-Arbeitspreis NT"""
    hoechstpreis_h_t: Optional["Preis"] = None
    """Höchstpreis für den Durchschnitts-Arbeitspreis HT"""
    mindestpreis: Optional["Preis"] = None
    """Mindestpreis für den Durchschnitts-Arbeitspreis"""
    zusatzpreise: Optional[list["Tarifpreis"]] = None
    """Liste mit zusätzlichen Preisen, beispielsweise Messpreise und/oder Leistungspreise"""



# pylint: disable=missing-module-docstring
from bo4e.enum.strenum import StrEnum


class Befestigungsart(StrEnum):
    """
    Befestigungsart von Zählern
    """

    STECKTECHNIK = "STECKTECHNIK"
    """STECKTECHNIK"""
    DREIPUNKT = "DREIPUNKT"
    """DREIPUNKT"""
    HUTSCHIENE = "HUTSCHIENE"
    """HUTSCHIENE"""
    EINSTUTZEN = "EINSTUTZEN"
    """EINSTUTZEN"""
    ZWEISTUTZEN = "ZWEISTUTZEN"
    """ZWEISTUTZEN"""



"""
Contains PreisblattDienstleistung class
"""

from typing import TYPE_CHECKING, Annotated, Literal, Optional

from pydantic import Field

from ..enum.botyp import BoTyp
from ..utils import postprocess_docstring
from .preisblatt import Preisblatt

if TYPE_CHECKING:
    from ..bo.geraet import Geraet
    from ..enum.bilanzierungsmethode import Bilanzierungsmethode
    from ..enum.dienstleistungstyp import Dienstleistungstyp


# pylint: disable=too-few-public-methods


@postprocess_docstring
class PreisblattDienstleistung(Preisblatt):
    """
    Variante des Preisblattmodells zur Abbildung der Preise für wahlfreie Dienstleistungen

    .. raw:: html

        <object data="../_static/images/bo4e/bo/PreisblattDienstleistung.svg" type="image/svg+xml"></object>

    .. HINT::
        `PreisblattDienstleistung JSON Schema <https://json-schema.app/view/%23?url=https://raw.githubusercontent.com/BO4E/BO4E-Schemas/{__gh_version__}/src/bo4e_schemas/bo/PreisblattDienstleistung.json>`_

    """

    typ: Annotated[Literal[BoTyp.PREISBLATTDIENSTLEISTUNG], Field(alias="_typ")] = (
        BoTyp.PREISBLATTDIENSTLEISTUNG  # type: ignore[assignment]
    )
    # required attributes (additional to those of Preisblatt)
    bilanzierungsmethode: Optional["Bilanzierungsmethode"] = None
    """Die Preise gelten für Marktlokationen der angebebenen Bilanzierungsmethode"""
    basisdienstleistung: Optional["Dienstleistungstyp"] = None
    """Dienstleistung, für die der Preis abgebildet wird, z.B. Sperrung/Entsperrung"""

    geraetedetails: Optional["Geraet"] = None
    """Hier kann der Preis auf bestimmte Geräte eingegrenzt werden. Z.B. auf die Zählergröße"""

    inklusive_dienstleistungen: Optional[list["Dienstleistungstyp"]] = None
    """Weitere Dienstleistungen, die im Preis enthalten sind"""



# pylint:disable=missing-module-docstring

from bo4e.enum.strenum import StrEnum


class Energierichtung(StrEnum):
    """
    Spezifiziert die Energierichtung einer Markt- und/oder Messlokation
    """

    AUSSP = "AUSSP"
    """Ausspeisung"""
    EINSP = "EINSP"
    """Einspeisung"""
